"""
Make sure the frame variable -g, -a, and -l flags work.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import os
import shutil
import time


class TestFrameVar(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def test_frame_var(self):
        self.build()
        self.do_test()

    def do_test(self):
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.c")
        )

        command_result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()

        # Ensure --regex can find globals if it is the very first frame var command.
        self.expect("frame var --regex g_", substrs=["g_var"])

        # Ensure the requested scope is respected:
        self.expect(
            "frame var --regex argc --no-args",
            error=True,
            substrs=["no variables matched the regular expression 'argc'"],
        )

        # Just get args:
        result = interp.HandleCommand("frame var -l", command_result)
        self.assertEqual(
            result, lldb.eReturnStatusSuccessFinishResult, "frame var -a didn't succeed"
        )
        output = command_result.GetOutput()
        self.assertIn("argc", output, "Args didn't find argc")
        self.assertIn("argv", output, "Args didn't find argv")
        self.assertNotIn("test_var", output, "Args found a local")
        self.assertNotIn("g_var", output, "Args found a global")

        value_list = command_result.GetValues(lldb.eNoDynamicValues)
        self.assertGreaterEqual(value_list.GetSize(), 2)
        value_names = []
        for value in value_list:
            value_names.append(value.GetName())
        self.assertIn("argc", value_names)
        self.assertIn("argv", value_names)

        # Just get locals:
        result = interp.HandleCommand("frame var -a", command_result)
        self.assertEqual(
            result, lldb.eReturnStatusSuccessFinishResult, "frame var -a didn't succeed"
        )
        output = command_result.GetOutput()
        self.assertNotIn("argc", output, "Locals found argc")
        self.assertNotIn("argv", output, "Locals found argv")
        self.assertIn("test_var", output, "Locals didn't find test_var")
        self.assertNotIn("g_var", output, "Locals found a global")

        # Get the file statics:
        result = interp.HandleCommand("frame var -l -a -g", command_result)
        self.assertEqual(
            result, lldb.eReturnStatusSuccessFinishResult, "frame var -a didn't succeed"
        )
        output = command_result.GetOutput()
        self.assertNotIn("argc", output, "Globals found argc")
        self.assertNotIn("argv", output, "Globals found argv")
        self.assertNotIn("test_var", output, "Globals found test_var")
        self.assertIn("g_var", output, "Globals didn't find g_var")

    def check_frame_variable_errors(self, thread, error_strings):
        command_result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()
        result = interp.HandleCommand("frame variable", command_result)
        self.assertEqual(
            result, lldb.eReturnStatusFailed, "frame var succeeded unexpectedly"
        )
        command_error = command_result.GetError()

        frame = thread.GetFrameAtIndex(0)
        var_list = frame.GetVariables(True, True, False, True)
        self.assertEqual(var_list.GetSize(), 0)
        api_error = var_list.GetError()
        api_error_str = api_error.GetCString()

        for s in error_strings:
            self.assertIn(s, command_error)
        for s in error_strings:
            self.assertIn(s, api_error_str)

        # Check the structured error data.
        data = api_error.GetErrorData()
        version = data.GetValueForKey("version")
        self.assertEqual(version.GetIntegerValue(), 1)
        err_ty = data.GetValueForKey("type")
        self.assertEqual(err_ty.GetIntegerValue(), lldb.eErrorTypeGeneric)
        message = str(data.GetValueForKey("errors").GetItemAtIndex(0))
        for s in error_strings:
            self.assertIn(s, message)

    @skipIfRemote
    @skipUnlessDarwin
    def test_darwin_dwarf_missing_obj(self):
        """
        Test that if we build a binary with DWARF in .o files and we remove
        the .o file for main.cpp, that we get an appropriate error when we
        do 'frame variable' that explains why we aren't seeing variables.
        """
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_obj = self.getBuildArtifact("main.o")
        # Delete the main.o file that contains the debug info so we force an
        # error when we run to main and try to get variables
        os.unlink(main_obj)

        # We have to set a named breakpoint because we don't have any debug info
        # because we deleted the main.o file.
        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(self, "main")
        error_strings = [
            'debug map object file "',
            'main.o" containing debug info does not exist, debug info will not be loaded',
        ]
        self.check_frame_variable_errors(thread, error_strings)

    @skipIfRemote
    @skipUnlessDarwin
    def test_darwin_dwarf_obj_mod_time_mismatch(self):
        """
        Test that if we build a binary with DWARF in .o files and we update
        the mod time of the .o file for main.cpp, that we get an appropriate
        error when we do 'frame variable' that explains why we aren't seeing
        variables.
        """
        self.build(debug_info="dwarf")
        exe = self.getBuildArtifact("a.out")
        main_obj = self.getBuildArtifact("main.o")

        # Set the modification time for main.o file to the current time after
        # sleeping for 2 seconds. This ensures the modification time will have
        # changed and will not match the modification time in the debug map and
        # force an error when we run to main and try to get variables
        time.sleep(2)
        os.utime(main_obj, None)

        # We have to set a named breakpoint because we don't have any debug info
        # because we deleted the main.o file since the mod times don't match
        # and debug info won't be loaded
        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(self, "main")

        error_strings = [
            'debug map object file "',
            'main.o" changed (actual: 0x',
            ", debug map: 0x",
            ") since this executable was linked, debug info will not be loaded",
        ]
        self.check_frame_variable_errors(thread, error_strings)

    @skipIfRemote
    @skipIfWindows  # Windows can't set breakpoints by name 'main' in this case.
    def test_gline_tables_only(self):
        """
        Test that if we build a binary with "-gline-tables-only" that we can
        set a file and line breakpoint successfully, and get an error
        letting us know that this build option was enabled when trying to
        read variables.
        """
        self.build(dictionary={"CFLAGS_EXTRAS": "-gline-tables-only"})
        exe = self.getBuildArtifact("a.out")

        # We have to set a named breakpoint because we don't have any debug info
        # because we deleted the main.o file since the mod times don't match
        # and debug info won't be loaded
        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(self, "main")
        error_strings = [
            "no variable information is available in debug info for this compile unit"
        ]
        self.check_frame_variable_errors(thread, error_strings)

    @skipUnlessPlatform(["linux", "freebsd"])
    @add_test_categories(["dwo"])
    def test_fission_missing_dwo(self):
        """
        Test that if we build a binary with "-gsplit-dwarf" that we can
        set a file and line breakpoint successfully, and get an error
        letting us know we were unable to load the .dwo file.
        """
        self.build(dictionary={"CFLAGS_EXTRAS": "-gsplit-dwarf"})
        exe = self.getBuildArtifact("a.out")
        main_dwo = self.getBuildArtifact("main.dwo")

        self.assertTrue(
            os.path.exists(main_dwo), 'Make sure "%s" file exists' % (main_dwo)
        )
        # Delete the main.dwo file that contains the debug info so we force an
        # error when we run to main and try to get variables.
        os.unlink(main_dwo)

        # We have to set a named breakpoint because we don't have any debug info
        # because we deleted the main.o file since the mod times don't match
        # and debug info won't be loaded
        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(self, "main")
        error_strings = [
            'unable to locate .dwo debug file "',
            'main.dwo" for skeleton DIE 0x',
        ]
        self.check_frame_variable_errors(thread, error_strings)

    @skipUnlessPlatform(["linux", "freebsd"])
    @add_test_categories(["dwo"])
    def test_fission_invalid_dwo_objectfile(self):
        """
        Test that if we build a binary with "-gsplit-dwarf" that we can
        set a file and line breakpoint successfully, and get an error
        letting us know we were unable to load the .dwo file because it
        existed, but it wasn't a valid object file.
        """
        self.build(dictionary={"CFLAGS_EXTRAS": "-gsplit-dwarf"})
        exe = self.getBuildArtifact("a.out")
        main_dwo = self.getBuildArtifact("main.dwo")

        self.assertTrue(
            os.path.exists(main_dwo), 'Make sure "%s" file exists' % (main_dwo)
        )
        # Overwrite the main.dwo with the main.c source file so that the .dwo
        # file exists, but it isn't a valid object file as there is an error
        # for this case.
        shutil.copyfile(self.getSourcePath("main.c"), main_dwo)

        # We have to set a named breakpoint because we don't have any debug info
        # because we deleted the main.o file since the mod times don't match
        # and debug info won't be loaded
        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(self, "main")
        error_strings = [
            'unable to load object file for .dwo debug file "'
            'main.dwo" for unit DIE 0x',
        ]
        self.check_frame_variable_errors(thread, error_strings)



"""
Tests navigating in the multiline expression editor.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestCase(PExpectTest):
    arrow_up = "\033[A"
    arrow_down = "\033[B"

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfEditlineSupportMissing
    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr48316")
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])  # Randomly fails on buildbot
    def test_nav_arrow_up(self):
        """Tests that we can navigate back to the previous line with the up arrow"""
        self.launch()

        # Start multiline expression mode by just running 'expr'
        self.child.sendline("expr")
        self.child.expect_exact("terminate with an empty line to evaluate")
        # Create a simple integer expression '123' and press enter.
        self.child.send("123\n")
        # We should see the prompt for the second line of our expression.
        self.child.expect_exact("2: ")
        # Go back to the first line and change 123 to 124.
        # Then press enter twice to evaluate our expression.
        self.child.send(self.arrow_up + "\b4\n\n")
        # The result of our expression should be 124 (our edited expression)
        # and not 123 (the one we initially typed).
        self.child.expect_exact("(int) $0 = 124")

        self.quit()

    @skipIfAsan
    @skipIfEditlineSupportMissing
    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr48316")
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])  # Randomly fails on buildbot
    def test_nav_arrow_down(self):
        """Tests that we can navigate to the next line with the down arrow"""
        self.launch()

        # Start multiline expression mode by just running 'expr'
        self.child.sendline("expr")
        self.child.expect_exact("terminate with an empty line to evaluate")
        # Create a simple integer expression '111' and press enter.
        self.child.send("111\n")
        # We should see the prompt for the second line of our expression.
        self.child.expect_exact("2: ")
        # Create another simple integer expression '222'.
        self.child.send("222")
        # Go back to the first line and change '111' to '111+' to make
        # an addition operation that spans two lines. We need to go up to
        # test that we can go back down again.
        self.child.send(self.arrow_up + "+")
        # Go back down to our second line and change '222' to '223'
        # so that the full expression is now '111+\n223'.
        # Then press enter twice to evaluate the expression.
        self.child.send(self.arrow_down + "\b3\n\n")
        # The result of our expression '111 + 223' should be '334'.
        # If the expression is '333' then arrow down failed to get
        # us back to the second line.
        self.child.expect_exact("(int) $0 = 334")

        self.quit()

    @skipIfAsan
    @skipIfEditlineSupportMissing
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])  # Randomly fails on buildbot
    def test_nav_arrow_up_empty(self):
        """
        Tests that navigating with the up arrow doesn't crash and skips
        empty history entries.
        """
        self.launch()

        # Create a real history entry '456' and then follow up with an
        # empty entry (that shouldn't be saved).
        self.child.sendline("expr")
        self.child.expect_exact("terminate with an empty line to evaluate")
        self.child.send("456\n\n")
        self.expect_prompt()

        self.child.sendline("expr")
        self.child.expect_exact("terminate with an empty line to evaluate")
        self.child.send("\n")
        self.expect_prompt()

        # The up arrow should recall the actual history entry and not the
        # the empty entry (as that one shouldn't have been saved).
        self.child.sendline("expr")
        self.child.expect_exact("terminate with an empty line to evaluate")
        self.child.send(self.arrow_up)
        self.child.expect_exact("456")
        self.child.send("\n\n")
        self.expect_prompt()

        self.quit()



"""
Test Debugger APIs.
"""

import lldb

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DebuggerAPITestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def test_debugger_api_boundary_condition(self):
        """Exercise SBDebugger APIs with boundary conditions."""
        self.dbg.HandleCommand(None)
        self.dbg.SetDefaultArchitecture(None)
        self.dbg.GetScriptingLanguage(None)
        self.dbg.CreateTarget(None)
        self.dbg.CreateTarget(None, None, None, True, lldb.SBError())
        self.dbg.CreateTargetWithFileAndTargetTriple(None, None)
        self.dbg.CreateTargetWithFileAndArch(None, None)
        self.dbg.FindTargetWithFileAndArch(None, None)
        self.dbg.SetInternalVariable(None, None, None)
        self.dbg.GetInternalVariableValue(None, None)
        # FIXME (filcab): We must first allow for the swig bindings to know if
        # a Python callback is set. (Check python-typemaps.swig)
        # self.dbg.SetLoggingCallback(None)
        self.dbg.SetPrompt(None)
        self.dbg.SetCurrentPlatform(None)
        self.dbg.SetCurrentPlatformSDKRoot(None)

        fresh_dbg = lldb.SBDebugger()
        self.assertEqual(len(fresh_dbg), 0)

    def test_debugger_delete_invalid_target(self):
        """SBDebugger.DeleteTarget() should not crash LLDB given and invalid target."""
        target = lldb.SBTarget()
        self.assertFalse(target.IsValid())
        self.dbg.DeleteTarget(target)

    def test_debugger_internal_variables(self):
        """Ensure that SBDebugger reachs the same instance of properties
        regardless CommandInterpreter's context initialization"""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        property_name = "target.process.memory-cache-line-size"

        def get_cache_line_size():
            value_list = lldb.SBStringList()
            value_list = self.dbg.GetInternalVariableValue(
                property_name, self.dbg.GetInstanceName()
            )

            self.assertEqual(value_list.GetSize(), 1)
            self.assertEqual(value_list[0], value_list.GetStringAtIndex(0))
            try:
                return int(value_list.GetStringAtIndex(0))
            except ValueError as error:
                self.fail("Value is not a number: " + error)

        # Get global property value while there are no processes.
        global_cache_line_size = get_cache_line_size()

        # Run a process via SB interface. CommandInterpreter's execution context
        # remains empty.
        error = lldb.SBError()
        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetLaunchFlags(lldb.eLaunchFlagStopAtEntry)
        process = target.Launch(launch_info, error)
        self.assertTrue(process, PROCESS_IS_VALID)

        # This should change the value of a process's local property.
        new_cache_line_size = global_cache_line_size + 512
        error = self.dbg.SetInternalVariable(
            property_name, str(new_cache_line_size), self.dbg.GetInstanceName()
        )
        self.assertSuccess(error, property_name + " value was changed successfully")

        # Check that it was set actually.
        self.assertEqual(get_cache_line_size(), new_cache_line_size)

        # Run any command to initialize CommandInterpreter's execution context.
        self.runCmd("target list")

        # Test the local property again, is it set to new_cache_line_size?
        self.assertEqual(get_cache_line_size(), new_cache_line_size)

    @expectedFailureAll(
        remote=True,
        bugnumber="github.com/llvm/llvm-project/issues/92419",
    )
    def test_CreateTarget_platform(self):
        exe = self.getBuildArtifact("a.out")
        self.yaml2obj("elf.yaml", exe)
        error = lldb.SBError()
        target1 = self.dbg.CreateTarget(exe, None, "remote-linux", False, error)
        self.assertSuccess(error)
        platform1 = target1.GetPlatform()
        platform1.SetWorkingDirectory("/foo/bar")

        # Reuse a platform if it matches the currently selected one...
        target2 = self.dbg.CreateTarget(exe, None, "remote-linux", False, error)
        self.assertSuccess(error)
        platform2 = target2.GetPlatform()
        self.assertTrue(
            platform2.GetWorkingDirectory().endswith("bar"),
            platform2.GetWorkingDirectory(),
        )

        # ... but create a new one if it doesn't.
        self.dbg.SetSelectedPlatform(lldb.SBPlatform("remote-windows"))
        target3 = self.dbg.CreateTarget(exe, None, "remote-linux", False, error)
        self.assertSuccess(error)
        platform3 = target3.GetPlatform()
        self.assertIsNone(platform3.GetWorkingDirectory())

    def test_CreateTarget_arch(self):
        exe = self.getBuildArtifact("a.out")
        if lldbplatformutil.getHostPlatform() == "linux":
            self.yaml2obj("macho.yaml", exe)
            arch = "x86_64-apple-macosx"
            expected_platform = "remote-macosx"
        else:
            self.yaml2obj("elf.yaml", exe)
            arch = "x86_64-pc-linux"
            expected_platform = "remote-linux"

        fbsd = lldb.SBPlatform("remote-freebsd")
        self.dbg.SetSelectedPlatform(fbsd)

        error = lldb.SBError()
        target1 = self.dbg.CreateTarget(exe, arch, None, False, error)
        self.assertSuccess(error)
        platform1 = target1.GetPlatform()
        self.assertEqual(platform1.GetName(), expected_platform)
        platform1.SetWorkingDirectory("/foo/bar")

        # Reuse a platform even if it is not currently selected.
        self.dbg.SetSelectedPlatform(fbsd)
        target2 = self.dbg.CreateTarget(exe, arch, None, False, error)
        self.assertSuccess(error)
        platform2 = target2.GetPlatform()
        self.assertEqual(platform2.GetName(), expected_platform)
        self.assertTrue(
            platform2.GetWorkingDirectory().endswith("bar"),
            platform2.GetWorkingDirectory(),
        )

    def test_SetDestroyCallback(self):
        destroy_dbg_id = None

        def foo(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal destroy_dbg_id
            destroy_dbg_id = dbg_id

        self.dbg.SetDestroyCallback(foo)

        original_dbg_id = self.dbg.GetID()
        self.dbg.Destroy(self.dbg)
        self.assertEqual(destroy_dbg_id, original_dbg_id)

    def test_AddDestroyCallback(self):
        original_dbg_id = self.dbg.GetID()
        called = []

        def foo(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal called
            called += [('foo', dbg_id)]

        def bar(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal called
            called += [('bar', dbg_id)]

        token_foo = self.dbg.AddDestroyCallback(foo)
        token_bar = self.dbg.AddDestroyCallback(bar)
        self.dbg.Destroy(self.dbg)

        # Should call both `foo()` and `bar()`.
        self.assertEqual(called, [
            ('foo', original_dbg_id),
            ('bar', original_dbg_id),
        ])

    def test_RemoveDestroyCallback(self):
        original_dbg_id = self.dbg.GetID()
        called = []

        def foo(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal called
            called += [('foo', dbg_id)]

        def bar(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal called
            called += [('bar', dbg_id)]

        token_foo = self.dbg.AddDestroyCallback(foo)
        token_bar = self.dbg.AddDestroyCallback(bar)
        ret = self.dbg.RemoveDestroyCallback(token_foo)
        self.dbg.Destroy(self.dbg)

        # `Remove` should be successful
        self.assertTrue(ret)
        # Should only call `bar()`
        self.assertEqual(called, [('bar', original_dbg_id)])

    def test_RemoveDestroyCallback_invalid_token(self):
        original_dbg_id = self.dbg.GetID()
        magic_token_that_should_not_exist = 32413
        called = []

        def foo(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal called
            called += [('foo', dbg_id)]

        token_foo = self.dbg.AddDestroyCallback(foo)
        ret = self.dbg.RemoveDestroyCallback(magic_token_that_should_not_exist)
        self.dbg.Destroy(self.dbg)

        # `Remove` should be unsuccessful
        self.assertFalse(ret)
        # Should call `foo()`
        self.assertEqual(called, [('foo', original_dbg_id)])

    def test_HandleDestroyCallback(self):
        """
        Validates:
        1. AddDestroyCallback and RemoveDestroyCallback work during debugger destroy.
        2. HandleDestroyCallback invokes all callbacks in FIFO order.
        """
        original_dbg_id = self.dbg.GetID()
        events = []
        bar_token = None

        def foo(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal events
            events.append(('foo called', dbg_id))

        def bar(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal events
            events.append(('bar called', dbg_id))

        def add_foo(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal events
            events.append(('add_foo called', dbg_id))
            events.append(('foo token', self.dbg.AddDestroyCallback(foo)))

        def remove_bar(dbg_id):
            # Need nonlocal to modify closure variable.
            nonlocal events
            events.append(('remove_bar called', dbg_id))
            events.append(('remove bar ret', self.dbg.RemoveDestroyCallback(bar_token)))

        # Setup
        events.append(('add_foo token', self.dbg.AddDestroyCallback(add_foo)))
        bar_token = self.dbg.AddDestroyCallback(bar)
        events.append(('bar token', bar_token))
        events.append(('remove_bar token', self.dbg.AddDestroyCallback(remove_bar)))
        # Destroy
        self.dbg.Destroy(self.dbg)

        self.assertEqual(events, [
            # Setup
            ('add_foo token', 0), # add_foo should be added
            ('bar token', 1), # bar should be added
            ('remove_bar token', 2), # remove_bar should be added
            # Destroy
            ('add_foo called', original_dbg_id), # add_foo should be called
            ('foo token', 3), # foo should be added
            ('bar called', original_dbg_id), # bar should be called
            ('remove_bar called', original_dbg_id), # remove_bar should be called
            ('remove bar ret', False), # remove_bar should fail, because it's already invoked and removed
            ('foo called', original_dbg_id), # foo should be called
        ])

    def test_version(self):
        instance_str = self.dbg.GetVersionString()
        class_str = lldb.SBDebugger.GetVersionString()
        property_str = lldb.SBDebugger.version

        self.assertEqual(instance_str, class_str)
        self.assertEqual(class_str, property_str)

    def test_find_target_with_unique_id(self):
        """Test SBDebugger.FindTargetByGloballyUniqueID() functionality."""

        # Test with invalid ID - should return invalid target
        invalid_target = self.dbg.FindTargetByGloballyUniqueID(999999)
        self.assertFalse(invalid_target.IsValid())

        # Test with ID 0 - should return invalid target
        zero_target = self.dbg.FindTargetByGloballyUniqueID(0)
        self.assertFalse(zero_target.IsValid())

        # Build a real executable and create target with it
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target.IsValid())

        # Find the target using its unique ID
        unique_id = target.GetGloballyUniqueID()
        self.assertNotEqual(unique_id, lldb.LLDB_INVALID_GLOBALLY_UNIQUE_TARGET_ID)
        found_target = self.dbg.FindTargetByGloballyUniqueID(unique_id)
        self.assertTrue(found_target.IsValid())
        self.assertEqual(
            self.dbg.GetIndexOfTarget(target), self.dbg.GetIndexOfTarget(found_target)
        )
        self.assertEqual(found_target.GetGloballyUniqueID(), unique_id)

    def test_target_unique_id_uniqueness(self):
        """Test that Target.GetGloballyUniqueID() returns unique values across multiple targets."""

        # Create multiple targets and verify they all have unique IDs
        self.build()
        exe = self.getBuildArtifact("a.out")
        targets = []
        unique_ids = set()

        for i in range(10):
            target = self.dbg.CreateTarget(exe)
            self.assertTrue(target.IsValid())

            unique_id = target.GetGloballyUniqueID()
            self.assertNotEqual(unique_id, 0)

            # Verify this ID hasn't been used before
            self.assertNotIn(
                unique_id, unique_ids, f"Duplicate unique ID found: {unique_id}"
            )

            unique_ids.add(unique_id)
            targets.append(target)

        # Verify all targets can still be found by their IDs
        for target in targets:
            unique_id = target.GetGloballyUniqueID()
            found = self.dbg.FindTargetByGloballyUniqueID(unique_id)
            self.assertTrue(found.IsValid())
            self.assertEqual(found.GetGloballyUniqueID(), unique_id)

    def test_target_unique_id_uniqueness_after_deletion(self):
        """Test finding targets have unique ID after target deletion."""
        # Create two targets
        self.build()
        exe = self.getBuildArtifact("a.out")
        target1 = self.dbg.CreateTarget(exe)
        target2 = self.dbg.CreateTarget(exe)
        self.assertTrue(target1.IsValid())
        self.assertTrue(target2.IsValid())

        unique_id1 = target1.GetGloballyUniqueID()
        unique_id2 = target2.GetGloballyUniqueID()
        self.assertNotEqual(unique_id1, 0)
        self.assertNotEqual(unique_id2, 0)
        self.assertNotEqual(unique_id1, unique_id2)

        # Verify we can find them initially
        found_target1 = self.dbg.FindTargetByGloballyUniqueID(unique_id1)
        found_target2 = self.dbg.FindTargetByGloballyUniqueID(unique_id2)
        self.assertTrue(found_target1.IsValid())
        self.assertTrue(found_target2.IsValid())
        target2_index = self.dbg.GetIndexOfTarget(target2)

        # Delete target 2
        deleted = self.dbg.DeleteTarget(target2)
        self.assertTrue(deleted)

        # Try to find the deleted target - should not be found
        not_found_target = self.dbg.FindTargetByGloballyUniqueID(unique_id2)
        self.assertFalse(not_found_target.IsValid())

        # Create a new target
        target3 = self.dbg.CreateTarget(exe)
        self.assertTrue(target3.IsValid())
        # Target list index of target3 should be the same as target2's
        # since it was deleted, but it should have a distinct unique ID
        target3_index = self.dbg.GetIndexOfTarget(target3)
        unique_id3 = target3.GetGloballyUniqueID()
        self.assertEqual(target3_index, target2_index)
        self.assertNotEqual(unique_id3, unique_id2)
        self.assertNotEqual(unique_id3, unique_id1)
        # Make sure we can find the new target
        found_target3 = self.dbg.FindTargetByGloballyUniqueID(
            target3.GetGloballyUniqueID()
        )
        self.assertTrue(found_target3.IsValid())

    def test_target_globally_unique_id_across_debuggers(self):
        """Test that target IDs are globally unique across multiple debuggers."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create two debuggers with targets each
        debugger1 = lldb.SBDebugger.Create()
        debugger2 = lldb.SBDebugger.Create()
        self.addTearDownHook(lambda: lldb.SBDebugger.Destroy(debugger1))
        self.addTearDownHook(lambda: lldb.SBDebugger.Destroy(debugger2))

        # Create 2 targets per debugger
        targets_d1 = [debugger1.CreateTarget(exe), debugger1.CreateTarget(exe)]
        targets_d2 = [debugger2.CreateTarget(exe), debugger2.CreateTarget(exe)]
        targets = targets_d1 + targets_d2

        # Get all IDs and verify they're unique
        ids = [target.GetGloballyUniqueID() for target in targets]
        self.assertEqual(
            len(set(ids)), len(ids), f"IDs should be globally unique: {ids}"
        )
        self.assertTrue(
            all(uid != lldb.LLDB_INVALID_GLOBALLY_UNIQUE_TARGET_ID for uid in ids),
            "All IDs should be valid",
        )

        # Verify targets can be found by their IDs in respective debuggers
        for debugger, target_pair in [
            (debugger1, targets[:2]),
            (debugger2, targets[2:]),
        ]:
            for target in target_pair:
                found = debugger.FindTargetByGloballyUniqueID(
                    target.GetGloballyUniqueID()
                )
                self.assertTrue(
                    found.IsValid(), "Target should be found by its unique ID"
                )
                self.assertEqual(
                    found.GetGloballyUniqueID(), target.GetGloballyUniqueID()
                )



"""
Test lldb-dap launch request.
"""

import lldbdap_testcase
import tempfile


class TestDAP_launch_stdio_redirection(lldbdap_testcase.DAPTestCaseBase):
    """
    Test stdio redirection.
    """

    def test(self):
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")

        with tempfile.NamedTemporaryFile("rt") as f:
            self.launch_and_configurationDone(program, stdio=[None, f.name])
            self.verify_process_exited()
            lines = f.readlines()
            self.assertIn(
                program, lines[0], "make sure program path is in first argument"
            )



"""
Test that the process continues running after we detach from it.
"""

import lldb
import time
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DetachResumesTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(
        oslist=["windows"],
        bugnumber="github.com/llvm/llvm-project/issues/144891",
    )
    def test_detach_resumes(self):
        self.build()
        exe = self.getBuildArtifact()

        # The inferior will use this file to let us know it is ready to be
        # attached.
        sync_file_path = lldbutil.append_to_process_working_directory(
            self, "sync_file_%d" % (int(time.time()))
        )

        # And this one to let us know it is running after we've detached from
        # it.
        exit_file_path = lldbutil.append_to_process_working_directory(
            self, "exit_file_%d" % (int(time.time()))
        )

        popen = self.spawnSubprocess(
            self.getBuildArtifact(exe), [sync_file_path, exit_file_path]
        )
        lldbutil.wait_for_file_on_target(self, sync_file_path)

        self.runCmd("process attach -p " + str(popen.pid))

        # Set a breakpoint at a place that will be called by multiple threads
        # simultaneously. On systems (e.g. linux) where the debugger needs to
        # send signals to suspend threads, these signals will race with threads
        # hitting the breakpoint (and stopping on their own).
        bpno = lldbutil.run_break_set_by_symbol(self, "break_here")

        # And let the inferior know it can call the function.
        self.runCmd("expr -- wait_for_debugger_flag = false")

        self.runCmd("continue")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Detach, the process should keep running after this, and not be stopped
        # by the signals that the debugger may have used to suspend the threads.
        self.runCmd("detach")

        lldbutil.wait_for_file_on_target(self, exit_file_path)



"""Test breakpoint on a class constructor; and variable list the this object."""


import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ClassTypesTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.cpp.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Test 'frame variable this' when stopped on a class constructor."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break on the ctor function of class C.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=-1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The test suite sometimes shows that the process has exited without stopping.
        #
        # CC=clang ./dotest.py -v -t class_types
        # ...
        # Process 76604 exited with status = 0 (0x00000000)
        self.runCmd("process status")

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # We should be stopped on the ctor function of class C.
        self.expect(
            "frame variable --show-types this",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["C *", " this = "],
        )

    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Use Python APIs to create a breakpoint by (filespec, line)."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        filespec = target.GetExecutable()
        self.assertTrue(filespec, VALID_FILESPEC)

        fsDir = os.path.normpath(filespec.GetDirectory())
        fsFile = filespec.GetFilename()

        self.assertTrue(
            fsDir == os.path.dirname(self.getBuildArtifact()) and fsFile == "a.out",
            "FileSpec matches the executable",
        )

        bpfilespec = lldb.SBFileSpec("main.cpp", False)

        breakpoint = target.BreakpointCreateByLocation(bpfilespec, self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Verify the breakpoint just created.
        self.expect(
            str(breakpoint),
            BREAKPOINT_CREATED,
            exe=False,
            substrs=["main.cpp", str(self.line)],
        )

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.Launch() failed")

        if process.GetState() != lldb.eStateStopped:
            self.fail(
                "Process should be in the 'stopped' state, "
                "instead the actual state is: '%s'"
                % lldbutil.state_type_to_str(process.GetState())
            )

        # The stop reason of the thread should be breakpoint.
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # The filename of frame #0 should be 'main.cpp' and the line number
        # should be 93.
        self.expect(
            "%s:%d"
            % (lldbutil.get_filenames(thread)[0], lldbutil.get_line_numbers(thread)[0]),
            "Break correctly at main.cpp:%d" % self.line,
            exe=False,
            startstr="main.cpp:",
        )
        # clang compiled code reported main.cpp:94?
        # startstr = "main.cpp:93")

        # We should be stopped on the breakpoint with a hit count of 1.
        self.assertEqual(breakpoint.GetHitCount(), 1, BREAKPOINT_HIT_ONCE)

        process.Continue()

    def test_with_expr_parser(self):
        """Test 'frame variable this' and 'expr this' when stopped inside a constructor."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # rdar://problem/8516141
        # Is this a case of clang (116.1) generating bad debug info?
        #
        # Break on the ctor function of class C.
        # self.expect("breakpoint set -M C", BREAKPOINT_CREATED,
        #    startstr = "Breakpoint created: 1: name = 'C'")

        # Make the test case more robust by using line number to break,
        # instead.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=-1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # Continue on inside the ctor() body...
        self.runCmd("register read pc")
        self.runCmd("thread step-over")

        # Verify that 'frame variable this' gets the data type correct.
        self.expect(
            "frame variable this", VARIABLES_DISPLAYED_CORRECTLY, substrs=["C *"]
        )

        # Verify that frame variable --show-types this->m_c_int behaves
        # correctly.
        self.runCmd("register read pc")
        self.runCmd("expr m_c_int")
        self.expect(
            "frame variable --show-types this->m_c_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(int) this->m_c_int = 66",
        )

        # Verify that 'expression this' gets the data type correct.
        self.expect("expression this", VARIABLES_DISPLAYED_CORRECTLY, substrs=["C *"])

        # rdar://problem/8430916
        # expr this->m_c_int returns an incorrect value
        #
        # Verify that expr this->m_c_int behaves correctly.
        self.expect(
            "expression this->m_c_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            patterns=[r"\(int\) \$[0-9]+ = 66"],
        )

    def test_with_constructor_name(self):
        """Test 'frame variable this' and 'expr this' when stopped inside a constructor."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        filespec = target.GetExecutable()
        self.assertTrue(filespec, VALID_FILESPEC)

        fsDir = os.path.normpath(filespec.GetDirectory())
        fsFile = filespec.GetFilename()

        self.assertTrue(
            fsDir == os.path.dirname(self.getBuildArtifact()) and fsFile == "a.out",
            "FileSpec matches the executable",
        )

        bpfilespec = lldb.SBFileSpec("main.cpp", False)

        breakpoint = target.BreakpointCreateByLocation(bpfilespec, self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Verify the breakpoint just created.
        self.expect(
            str(breakpoint),
            BREAKPOINT_CREATED,
            exe=False,
            substrs=["main.cpp", str(self.line)],
        )

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.Launch() failed")

        if process.GetState() != lldb.eStateStopped:
            self.fail(
                "Process should be in the 'stopped' state, "
                "instead the actual state is: '%s'"
                % lldbutil.state_type_to_str(process.GetState())
            )

        # The stop reason of the thread should be breakpoint.
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        frame = thread.frames[0]
        self.assertTrue(frame.IsValid(), "Got a valid frame.")

        self.assertIn("C::C", frame.name, "Constructor name includes class name.")



import lldb
import subprocess
import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestMSVCRTCException(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessPlatform(["windows"])
    @skipUnlessMSVC
    def test_msvc_runtime_checks(self):
        """Test that lldb prints MSVC's runtime checks exceptions as stop reasons."""

        src = os.path.join(self.getSourceDir(), "main.c")
        exe = os.path.join(self.getBuildDir(), "a.exe")

        result = subprocess.run(
            ["cl.exe", "/nologo", "/Od", "/Zi", "/MDd", "/RTC1", "/Fe" + exe, src],
            cwd=self.getBuildDir(),
            capture_output=True,
            text=True,
        )
        self.assertEqual(
            result.returncode,
            0,
            "Compilation failed:\n" + result.stdout + result.stderr,
        )

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target.IsValid(), "Could not create target")

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process.IsValid(), "Could not launch process")

        self.assertEqual(process.GetState(), lldb.eStateStopped)

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonException)
        self.assertIsNotNone(thread, "No thread stopped with exception stop reason")

        stop_description = thread.GetStopDescription(256)
        self.assertIn(
            "Run-time check failure",
            stop_description,
            "Stop reason does not mention run-time check failure",
        )
        self.assertIn(
            "variable 'x' is being used without being initialized",
            stop_description,
            "Stop reason does not mention uninitialized variable 'x'",
        )



"""
Make sure 'frame var' using DIL parser/evaluator works for bit fields.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil

import os
import shutil
import time


class TestFrameVarDILBitField(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_frame_var(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")
        self.expect_var_path("bf.a", value="1023")
        self.expect_var_path("bf.b", value="9")
        self.expect_var_path("bf.c", value="false")
        self.expect_var_path("bf.d", value="true")

        self.expect_var_path("abf.a", value="1023")
        self.expect_var_path("abf.b", value="'\\x0f'")
        self.expect_var_path("abf.c", value="3")

        # Perform an operation to ensure we actually read the value.
        # Address-of is not allowed for bit-fields.
        self.expect(
            "frame variable '&bf.a'",
            error=True,
            substrs=["'bf.a' doesn't have a valid address"],
        )



"""
A stress-test of sorts for LLDB's handling of threads in the inferior.

This test sets a breakpoint in the main thread where test parameters (numbers of
threads) can be adjusted, runs the inferior to that point, and modifies the
locals that control the event thread counts. This test also sets a breakpoint in
breakpoint_func (the function executed by each 'breakpoint' thread) and a
watchpoint on a global modified in watchpoint_func. The inferior is continued
until exit or a crash takes place, and the number of events seen by LLDB is
verified to match the expected number of events.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ConcurrentEventsBase(TestBase):
    # Concurrency is the primary test factor here, not debug info variants.
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        super(ConcurrentEventsBase, self).setUp()
        # Find the line number for our breakpoint.
        self.filename = "main.cpp"
        self.thread_breakpoint_line = line_number(
            self.filename, "// Set breakpoint here"
        )
        self.setup_breakpoint_line = line_number(
            self.filename, "// Break here and adjust num"
        )
        self.finish_breakpoint_line = line_number(
            self.filename, "// Break here and verify one thread is active"
        )

    def describe_threads(self):
        ret = []
        for x in self.inferior_process:
            id = x.GetIndexID()
            reason = x.GetStopReason()
            status = "stopped" if x.IsStopped() else "running"
            reason_str = lldbutil.stop_reason_to_str(reason)
            if reason == lldb.eStopReasonBreakpoint:
                bpid = x.GetStopReasonDataAtIndex(0)
                bp = self.inferior_target.FindBreakpointByID(bpid)
                reason_str = "%s hit %d times" % (
                    lldbutil.get_description(bp),
                    bp.GetHitCount(),
                )
            elif reason == lldb.eStopReasonWatchpoint:
                watchid = x.GetStopReasonDataAtIndex(0)
                watch = self.inferior_target.FindWatchpointByID(watchid)
                reason_str = "%s hit %d times" % (
                    lldbutil.get_description(watch),
                    watch.GetHitCount(),
                )
            elif reason == lldb.eStopReasonSignal:
                signals = self.inferior_process.GetUnixSignals()
                signal_name = signals.GetSignalAsCString(x.GetStopReasonDataAtIndex(0))
                reason_str = "signal %s" % signal_name

            location = "\t".join(
                [
                    lldbutil.get_description(x.GetFrameAtIndex(i))
                    for i in range(x.GetNumFrames())
                ]
            )
            ret.append(
                "thread %d %s due to %s at\n\t%s" % (id, status, reason_str, location)
            )
        return ret

    def add_breakpoint(self, line, descriptions):
        """Adds a breakpoint at self.filename:line and appends its description to descriptions, and
        returns the LLDB SBBreakpoint object.
        """

        bpno = lldbutil.run_break_set_by_file_and_line(
            self, self.filename, line, num_expected_locations=-1
        )
        bp = self.inferior_target.FindBreakpointByID(bpno)
        descriptions.append(": file = 'main.cpp', line = %d" % line)
        return bp

    def inferior_done(self):
        """Returns true if the inferior is done executing all the event threads (and is stopped at self.finish_breakpoint,
        or has terminated execution.
        """
        return (
            self.finish_breakpoint.GetHitCount() > 0
            or self.crash_count > 0
            or self.inferior_process.GetState() == lldb.eStateExited
        )

    def count_signaled_threads(self):
        count = 0
        for thread in self.inferior_process:
            if (
                thread.GetStopReason() == lldb.eStopReasonSignal
                and thread.GetStopReasonDataAtIndex(0)
                == self.inferior_process.GetUnixSignals().GetSignalNumberFromName(
                    "SIGUSR1"
                )
            ):
                count += 1
        return count

    def do_thread_actions(
        self,
        num_breakpoint_threads=0,
        num_signal_threads=0,
        num_watchpoint_threads=0,
        num_crash_threads=0,
        num_delay_breakpoint_threads=0,
        num_delay_signal_threads=0,
        num_delay_watchpoint_threads=0,
        num_delay_crash_threads=0,
    ):
        """Sets a breakpoint in the main thread where test parameters (numbers of threads) can be adjusted, runs the inferior
        to that point, and modifies the locals that control the event thread counts. Also sets a breakpoint in
        breakpoint_func (the function executed by each 'breakpoint' thread) and a watchpoint on a global modified in
        watchpoint_func. The inferior is continued until exit or a crash takes place, and the number of events seen by LLDB
        is verified to match the expected number of events.
        """
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Get the target
        self.inferior_target = self.dbg.GetSelectedTarget()

        expected_bps = []

        # Initialize all the breakpoints (main thread/aux thread)
        self.setup_breakpoint = self.add_breakpoint(
            self.setup_breakpoint_line, expected_bps
        )
        self.finish_breakpoint = self.add_breakpoint(
            self.finish_breakpoint_line, expected_bps
        )

        # Set the thread breakpoint
        if num_breakpoint_threads + num_delay_breakpoint_threads > 0:
            self.thread_breakpoint = self.add_breakpoint(
                self.thread_breakpoint_line, expected_bps
            )

        # Verify breakpoints
        self.expect(
            "breakpoint list -f",
            "Breakpoint locations shown correctly",
            substrs=expected_bps,
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Check we are at line self.setup_breakpoint
        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        # Initialize the (single) watchpoint on the global variable (g_watchme)
        if num_watchpoint_threads + num_delay_watchpoint_threads > 0:
            # The concurrent tests have multiple threads modifying a variable
            # with the same value.  The default "modify" style watchpoint will
            # only report this as 1 hit for all threads, because they all wrote
            # the same value.  The testsuite needs "write" style watchpoints to
            # get the correct number of hits reported.
            self.runCmd("watchpoint set variable -w write g_watchme")
            for w in self.inferior_target.watchpoint_iter():
                self.thread_watchpoint = w
                self.assertTrue(
                    "g_watchme" in str(self.thread_watchpoint),
                    "Watchpoint location not shown correctly",
                )

        # Get the process
        self.inferior_process = self.inferior_target.GetProcess()

        # We should be stopped at the setup site where we can set the number of
        # threads doing each action (break/crash/signal/watch)
        self.assertEqual(
            self.inferior_process.GetNumThreads(),
            1,
            "Expected to stop before any additional threads are spawned.",
        )

        self.runCmd("expr num_breakpoint_threads=%d" % num_breakpoint_threads)
        self.runCmd("expr num_crash_threads=%d" % num_crash_threads)
        self.runCmd("expr num_signal_threads=%d" % num_signal_threads)
        self.runCmd("expr num_watchpoint_threads=%d" % num_watchpoint_threads)

        self.runCmd(
            "expr num_delay_breakpoint_threads=%d" % num_delay_breakpoint_threads
        )
        self.runCmd("expr num_delay_crash_threads=%d" % num_delay_crash_threads)
        self.runCmd("expr num_delay_signal_threads=%d" % num_delay_signal_threads)
        self.runCmd(
            "expr num_delay_watchpoint_threads=%d" % num_delay_watchpoint_threads
        )

        # Continue the inferior so threads are spawned
        self.runCmd("continue")

        # Make sure we see all the threads. The inferior program's threads all synchronize with a pseudo-barrier; that is,
        # the inferior program ensures all threads are started and running
        # before any thread triggers its 'event'.
        num_threads = self.inferior_process.GetNumThreads()
        expected_num_threads = (
            num_breakpoint_threads
            + num_delay_breakpoint_threads
            + num_signal_threads
            + num_delay_signal_threads
            + num_watchpoint_threads
            + num_delay_watchpoint_threads
            + num_crash_threads
            + num_delay_crash_threads
            + 1
        )
        self.assertEqual(
            num_threads,
            expected_num_threads,
            "Expected to see %d threads, but seeing %d. Details:\n%s"
            % (expected_num_threads, num_threads, "\n\t".join(self.describe_threads())),
        )

        self.signal_count = self.count_signaled_threads()
        self.crash_count = len(
            lldbutil.get_crashed_threads(self, self.inferior_process)
        )

        # Run to completion (or crash)
        while not self.inferior_done():
            if self.TraceOn():
                self.runCmd("thread backtrace all")
            self.runCmd("continue")
            self.signal_count += self.count_signaled_threads()
            self.crash_count += len(
                lldbutil.get_crashed_threads(self, self.inferior_process)
            )

        if num_crash_threads > 0 or num_delay_crash_threads > 0:
            # Expecting a crash
            self.assertTrue(
                self.crash_count > 0,
                "Expecting at least one thread to crash. Details: %s"
                % "\t\n".join(self.describe_threads()),
            )

            # Ensure the zombie process is reaped
            self.runCmd("process kill")

        elif num_crash_threads == 0 and num_delay_crash_threads == 0:
            # There should be a single active thread (the main one) which hit
            # the breakpoint after joining
            self.assertEqual(
                1,
                self.finish_breakpoint.GetHitCount(),
                "Expected main thread (finish) breakpoint to be hit once",
            )

            # There should be a single active thread (the main one) which hit
            # the breakpoint after joining.  Depending on the pthread
            # implementation we may have a worker thread finishing the pthread_join()
            # after it has returned.  Filter the threads to only count those
            # with user functions on them from our test case file,
            # lldb/test/API/functionalities/thread/concurrent_events/main.cpp
            user_code_funcnames = [
                "breakpoint_func",
                "crash_func",
                "do_action_args",
                "dotest",
                "main",
                "register_signal_handler",
                "signal_func",
                "sigusr1_handler",
                "start_threads",
                "watchpoint_func",
            ]
            num_threads_with_usercode = 0
            for t in self.inferior_process.threads:
                thread_has_user_code = False
                for f in t.frames:
                    for funcname in user_code_funcnames:
                        if funcname in f.GetDisplayFunctionName():
                            thread_has_user_code = True
                            break

                    if thread_has_user_code:
                        break

                if thread_has_user_code:
                    num_threads_with_usercode += 1

            self.assertEqual(
                1,
                num_threads_with_usercode,
                "Expecting 1 thread but seeing %d. Details:%s"
                % (num_threads_with_usercode, "\n\t".join(self.describe_threads())),
            )
            self.runCmd("continue")

            # The inferior process should have exited without crashing
            self.assertEqual(
                0, self.crash_count, "Unexpected thread(s) in crashed state"
            )
            self.assertEqual(
                self.inferior_process.GetState(), lldb.eStateExited, PROCESS_EXITED
            )

            # Verify the number of actions took place matches expected numbers
            expected_breakpoint_threads = (
                num_delay_breakpoint_threads + num_breakpoint_threads
            )
            breakpoint_hit_count = (
                self.thread_breakpoint.GetHitCount()
                if expected_breakpoint_threads > 0
                else 0
            )
            self.assertEqual(
                expected_breakpoint_threads,
                breakpoint_hit_count,
                "Expected %d breakpoint hits, but got %d"
                % (expected_breakpoint_threads, breakpoint_hit_count),
            )

            expected_signal_threads = num_delay_signal_threads + num_signal_threads
            self.assertEqual(
                expected_signal_threads,
                self.signal_count,
                "Expected %d stops due to signal delivery, but got %d"
                % (expected_signal_threads, self.signal_count),
            )

            expected_watchpoint_threads = (
                num_delay_watchpoint_threads + num_watchpoint_threads
            )
            watchpoint_hit_count = (
                self.thread_watchpoint.GetHitCount()
                if expected_watchpoint_threads > 0
                else 0
            )
            self.assertEqual(
                expected_watchpoint_threads,
                watchpoint_hit_count,
                "Expected %d watchpoint hits, got %d"
                % (expected_watchpoint_threads, watchpoint_hit_count),
            )



"""
Test that we can call functions and use types
annotated (and thus mangled) with ABI tags.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AbiTagLookupTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipIfWindows
    @expectedFailureAll(debug_info=["dwarf", "gmodules", "dwo"])
    def test_abi_tag_lookup(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Break here", lldb.SBFileSpec("main.cpp", False)
        )

        # Qualified/unqualified lookup to templates in namespace
        self.expect_expr("operator<(b1, b2)", result_type="bool", result_value="true")
        self.expect_expr(
            "A::operator<(b1, b2)", result_type="bool", result_value="true"
        )
        self.expect_expr("b1 < b2", result_type="bool", result_value="true")

        # Qualified/unqualified lookup to templates with ABI tags in namespace
        self.expect_expr("operator>(b1, b2)", result_type="bool", result_value="true")
        self.expect_expr(
            "A::operator>(b1, b2)", result_type="bool", result_value="true"
        )
        self.expect_expr("b1 > b2", result_type="bool", result_value="true")

        # Call non-operator templates with ABI tags
        self.expect_expr("A::withAbiTagInNS(1, 1)", result_type="int", result_value="1")

        self.expect_expr(
            "A::withAbiTagInNS(1.0, 1.0)", result_type="int", result_value="2"
        )
        self.expect_expr("withAbiTagInNS(b1, b2)", result_type="int", result_value="2")
        self.expect_expr(
            "A::withAbiTagInNS(b1, b2)", result_type="int", result_value="2"
        )

        self.expect_expr("withAbiTag(b1, b2)", result_type="int", result_value="3")
        self.expect_expr("withAbiTag(0, 0)", result_type="int", result_value="-3")

        # Structures with ABI tags
        self.expect_expr("t.Value()", result_type="const int", result_value="4")
        self.expect_expr("tt.Value()", result_type="const int", result_value="5")

        self.expect_expr(
            "Tagged{.mem = 6}",
            result_type="Tagged",
            result_children=[ValueCheck(name="mem", value="6")],
        )

        # Inline namespaces with ABI tags
        self.expect_expr(
            "v1::withImplicitTag(Simple{.mem = 6})", result_type="int", result_value="6"
        )
        self.expect_expr(
            "withImplicitTag(Simple{.mem = 6})", result_type="int", result_value="6"
        )

        self.expect_expr(
            "v1::withImplicitTag(Tagged{.mem = 6})", result_type="int", result_value="6"
        )
        self.expect_expr(
            "withImplicitTag(Tagged{.mem = 6})", result_type="int", result_value="6"
        )



"""This file contains benchmarks for sparse tensors. In particular, it
contains benchmarks for both mlir sparse tensor dialect and numpy so that they
can be compared against each other.
"""
import ctypes
import numpy as np
import os
import re
import time

from mlir import ir
from mlir import runtime as rt
from mlir.dialects import func
from mlir.dialects.linalg.opdsl import lang as dsl
from mlir.execution_engine import ExecutionEngine

from common import create_sparse_np_tensor
from common import emit_timer_func
from common import emit_benchmark_wrapped_main_func
from common import get_kernel_func_from_module
from common import setup_passes


@dsl.linalg_structured_op
def matmul_dsl(
    A=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.K),
    B=dsl.TensorDef(dsl.T, dsl.S.K, dsl.S.N),
    C=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.N, output=True),
):
    """Helper function for mlir sparse matrix multiplication benchmark."""
    C[dsl.D.m, dsl.D.n] += A[dsl.D.m, dsl.D.k] * B[dsl.D.k, dsl.D.n]


def benchmark_sparse_mlir_multiplication():
    """Benchmark for mlir sparse matrix multiplication. Because its an
    MLIR benchmark we need to return both a `compiler` function and a `runner`
    function.
    """
    with ir.Context(), ir.Location.unknown():
        module = ir.Module.create()
        f64 = ir.F64Type.get()
        param1_type = ir.RankedTensorType.get([1000, 1500], f64)
        param2_type = ir.RankedTensorType.get([1500, 2000], f64)
        result_type = ir.RankedTensorType.get([1000, 2000], f64)
        with ir.InsertionPoint(module.body):

            @func.FuncOp.from_py_func(param1_type, param2_type, result_type)
            def sparse_kernel(x, y, z):
                return matmul_dsl(x, y, outs=[z])

    def compiler():
        with ir.Context(), ir.Location.unknown():
            kernel_func = get_kernel_func_from_module(module)
            timer_func = emit_timer_func()
            wrapped_func = emit_benchmark_wrapped_main_func(kernel_func, timer_func)
            main_module_with_benchmark = ir.Module.parse(
                str(timer_func) + str(wrapped_func) + str(kernel_func)
            )
            setup_passes(main_module_with_benchmark)
            c_runner_utils = os.getenv("MLIR_C_RUNNER_UTILS", "")
            assert os.path.exists(c_runner_utils), (
                f"{c_runner_utils} does not exist."
                f" Please pass a valid value for"
                f" MLIR_C_RUNNER_UTILS environment variable."
            )
            runner_utils = os.getenv("MLIR_RUNNER_UTILS", "")
            assert os.path.exists(runner_utils), (
                f"{runner_utils} does not exist."
                f" Please pass a valid value for MLIR_RUNNER_UTILS"
                f" environment variable."
            )

            engine = ExecutionEngine(
                main_module_with_benchmark,
                3,
                shared_libs=[c_runner_utils, runner_utils],
            )
            return engine.invoke

    def runner(engine_invoke):
        compiled_program_args = []
        for argument_type in [result_type, param1_type, param2_type, result_type]:
            argument_type_str = str(argument_type)
            dimensions_str = re.sub("<|>|tensor", "", argument_type_str)
            dimensions = [int(dim) for dim in dimensions_str.split("x")[:-1]]
            if argument_type == result_type:
                argument = np.zeros(dimensions, np.float64)
            else:
                argument = create_sparse_np_tensor(dimensions, 1000)
            compiled_program_args.append(
                ctypes.pointer(
                    ctypes.pointer(rt.get_ranked_memref_descriptor(argument))
                )
            )
        np_timers_ns = np.array([0], dtype=np.int64)
        compiled_program_args.append(
            ctypes.pointer(
                ctypes.pointer(rt.get_ranked_memref_descriptor(np_timers_ns))
            )
        )
        engine_invoke("main", *compiled_program_args)
        return int(np_timers_ns[0])

    return compiler, runner


def benchmark_np_matrix_multiplication():
    """Benchmark for numpy matrix multiplication. Because its a python
    benchmark, we don't have any `compiler` function returned. We just return
    the `runner` function.
    """

    def runner():
        argument1 = np.random.uniform(low=0.0, high=100.0, size=(1000, 1500))
        argument2 = np.random.uniform(low=0.0, high=100.0, size=(1500, 2000))
        start_time = time.time_ns()
        np.matmul(argument1, argument2)
        return time.time_ns() - start_time

    return None, runner



"""
Test Expression Parser regression test to ensure that we handle enums
correctly, in this case specifically std::vector of enums.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestVectorOfEnums(TestBase):
    SHARED_BUILD_TESTCASE = False

    @add_test_categories(["libc++"])
    def test_vector_of_enums(self):
        self.build(dictionary={"USE_LIBCPP": 1})

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect("expr v", substrs=["size=3", "[0] = a", "[1] = b", "[2] = c", "}"])



# On macOS, system python binaries like /usr/bin/python and $(xcrun -f python3)
# are shims. They do some light validation work and then spawn the "real" python
# binary. Find the "real" python by asking dyld -- sys.executable reports the
# wrong thing more often than not. This is also useful when we're running under
# a Homebrew python3 binary, which also appears to be some kind of shim.
def getDarwinRealPythonExecutable():
    import ctypes

    dyld = ctypes.cdll.LoadLibrary("/usr/lib/system/libdyld.dylib")
    namelen = ctypes.c_ulong(1024)
    name = ctypes.create_string_buffer(b"\000", namelen.value)
    dyld._NSGetExecutablePath(ctypes.byref(name), ctypes.byref(namelen))
    return name.value.decode("utf-8").strip()


print(getDarwinRealPythonExecutable())



"""Test that SBFrame::FindValue finds things but does not duplicate the entire variables list"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SBFrameFindValueTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_formatters_api(self):
        """Test that SBFrame::FindValue finds things but does not duplicate the entire variables list"""
        self.build()
        self.setTearDownCleanup()

        exe = self.getBuildArtifact("a.out")

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        breakpoint = target.BreakpointCreateBySourceRegex(
            "Set breakpoint here", lldb.SBFileSpec("main.cpp")
        )
        self.assertGreater(breakpoint.GetNumLocations(), 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be at our breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint)

        self.assertEqual(len(threads), 1)
        self.thread = threads[0]
        self.frame = self.thread.frames[0]
        self.assertTrue(self.frame, "Frame 0 is valid.")

        self.assertEqual(
            self.frame.GetVariables(True, True, False, True).GetSize(),
            2,
            "variable count is off",
        )
        self.assertFalse(
            self.frame.FindValue(
                "NoSuchThing",
                lldb.eValueTypeVariableArgument,
                lldb.eDynamicCanRunTarget,
            ).IsValid(),
            "found something that should not be here",
        )
        self.assertEqual(
            self.frame.GetVariables(True, True, False, True).GetSize(),
            2,
            "variable count is off after failed FindValue()",
        )
        self.assertTrue(
            self.frame.FindValue(
                "a", lldb.eValueTypeVariableArgument, lldb.eDynamicCanRunTarget
            ).IsValid(),
            "FindValue() didn't find an argument",
        )
        self.assertEqual(
            self.frame.GetVariables(True, True, False, True).GetSize(),
            2,
            "variable count is off after successful FindValue()",
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbplatform
from lldbsuite.test import lldbutil


class InlineSourceFilesTestCase(TestBase):
    @skipIf(compiler="gcc")
    @skipIf(compiler="clang", compiler_version=["<", "18.0"])
    # Fails on Windows for unknown reasons.
    @skipIfWindows
    def test(self):
        """Test DWARF inline source files."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "f")
        self.expect("list f", substrs=["This is inline source code"])



"""
Test that lldb watchpoint works for multiple threads.
"""

import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class WatchpointForMultipleThreadsTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    main_spec = lldb.SBFileSpec("main.cpp", False)

    @skipIfWindows  # This test is flaky on Windows
    def test_watchpoint_before_thread_start(self):
        """Test that we can hit a watchpoint we set before starting another thread"""
        self.do_watchpoint_test("Before running the thread")

    @skipIfWindows  # This test is flaky on Windows
    def test_watchpoint_after_thread_launch(self):
        """Test that we can hit a watchpoint we set after launching another thread"""
        self.do_watchpoint_test("After launching the thread")

    def test_watchpoint_after_thread_start(self):
        """Test that we can hit a watchpoint we set after another thread starts"""
        self.do_watchpoint_test("After running the thread")

    def do_watchpoint_test(self, line):
        self.build()
        lldbutil.run_to_source_breakpoint(self, line, self.main_spec)

        # Now let's set a write-type watchpoint for variable 'g_val'.
        self.expect(
            "watchpoint set variable -w write g_val",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 4", "type = w"],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v", substrs=["hit_count = 0"])

        self.runCmd("process continue")

        self.runCmd("thread list")
        if "stop reason = watchpoint" in self.res.GetOutput():
            # Good, we verified that the watchpoint works!
            self.runCmd("thread backtrace all")
        else:
            self.fail("The stop reason should be either break or watchpoint")

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be 1.
        self.expect("watchpoint list -v", substrs=["hit_count = 1"])

    def test_watchpoint_multiple_threads_wp_set_and_then_delete(self):
        """Test that lldb watchpoint works for multiple threads, and after the watchpoint is deleted, the watchpoint event should no longer fires."""
        self.build()
        self.setTearDownCleanup()

        lldbutil.run_to_source_breakpoint(
            self, "After running the thread", self.main_spec
        )

        # Now let's set a write-type watchpoint for variable 'g_val'.
        self.expect(
            "watchpoint set variable -w write g_val",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 4", "type = w"],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v", substrs=["hit_count = 0"])

        watchpoint_stops = 0
        while True:
            self.runCmd("process continue")
            self.runCmd("process status")
            if re.search("Process .* exited", self.res.GetOutput()):
                # Great, we are done with this test!
                break

            self.runCmd("thread list")
            if "stop reason = watchpoint" in self.res.GetOutput():
                self.runCmd("thread backtrace all")
                watchpoint_stops += 1
                if watchpoint_stops > 1:
                    self.fail("Watchpoint hits not supposed to exceed 1 by design!")
                # Good, we verified that the watchpoint works!  Now delete the
                # watchpoint.
                if self.TraceOn():
                    print(
                        "watchpoint_stops=%d at the moment we delete the watchpoint"
                        % watchpoint_stops
                    )
                self.runCmd("watchpoint delete 1")
                self.expect(
                    "watchpoint list -v", substrs=["No watchpoints currently set."]
                )
                continue
            else:
                self.fail("The stop reason should be either break or watchpoint")



"""
Test more expression command sequences with objective-c.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FoundationTestCaseNSArray(TestBase):
    def test_NSArray_expr_commands(self):
        """Test expression commands for NSArray."""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Break here for NSArray tests", lldb.SBFileSpec("main.m", False)
        )

        self.runCmd("thread backtrace")
        self.expect(
            "expression (int)[nil_mutable_array count]", patterns=[r"\(int\) \$.* = 0"]
        )
        self.expect("expression (int)[array1 count]", patterns=[r"\(int\) \$.* = 3"])
        self.expect("expression (int)[array2 count]", patterns=[r"\(int\) \$.* = 3"])
        self.expect("expression (int)array1.count", patterns=[r"\(int\) \$.* = 3"])
        self.expect("expression (int)array2.count", patterns=[r"\(int\) \$.* = 3"])
        self.runCmd("process continue")



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class ScriptDataFormatterTestCase(TestBase):
    def test_with_run_command(self):
        """Test data formatter commands."""
        self.build()
        self.data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def data_formatter_commands(self):
        """Test that that file and class static variables display correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Set the script here to ease the formatting
        script = "a = valobj.GetChildMemberWithName('integer'); a_val = a.GetValue(); str = 'Hello from Python, ' + a_val + ' time'; return str + ('!' if a_val == '1' else 's!');"

        self.runCmd('type summary add i_am_cool --python-script "%s"' % script)
        self.expect("type summary list i_am_cool", substrs=[script])

        self.expect("frame variable one", substrs=["Hello from Python", "1 time!"])

        self.expect("frame variable two", substrs=["Hello from Python", "4 times!"])

        self.runCmd("n")  # skip ahead to make values change

        self.expect(
            "frame variable three",
            substrs=["Hello from Python, 10 times!", "Hello from Python, 4 times!"],
        )

        self.runCmd("n")  # skip ahead to make values change

        self.expect("frame variable two", substrs=["Hello from Python", "1 time!"])

        script = "a = valobj.GetChildMemberWithName('integer'); a_val = a.GetValue(); str = 'int says ' + a_val; return str;"

        # Check that changes in the script are immediately reflected
        self.runCmd('type summary add i_am_cool --python-script "%s"' % script)

        self.expect("frame variable two", substrs=["int says 1"])

        self.expect("frame variable twoptr", substrs=["int says 1"])

        # Change the summary
        self.runCmd(
            'type summary add --summary-string "int says ${var.integer}, and float says ${var.floating}" i_am_cool'
        )

        self.expect("frame variable two", substrs=["int says 1", "and float says 2.71"])
        # Try it for pointers
        self.expect(
            "frame variable twoptr", substrs=["int says 1", "and float says 2.71"]
        )

        # Force a failure for pointers
        self.runCmd('type summary add i_am_cool -p --python-script "%s"' % script)

        self.expect(
            "frame variable twoptr", matching=False, substrs=["and float says 2.71"]
        )

        script = "return 'Python summary'"

        self.runCmd(
            'type summary add --name test_summary --python-script "%s"' % script
        )

        # attach the Python named summary to someone
        self.expect(
            "frame variable one --summary test_summary", substrs=["Python summary"]
        )

        # should not bind to the type
        self.expect("frame variable two", matching=False, substrs=["Python summary"])

        # and should not stick to the variable
        self.expect("frame variable one", matching=False, substrs=["Python summary"])

        self.runCmd('type summary add i_am_cool --summary-string "Text summary"')

        # should be temporary only
        self.expect("frame variable one", matching=False, substrs=["Python summary"])

        # use the type summary
        self.expect("frame variable two", substrs=["Text summary"])

        self.runCmd("n")  # skip ahead to make values change

        # both should use the type summary now
        self.expect("frame variable one", substrs=["Text summary"])

        self.expect("frame variable two", substrs=["Text summary"])

        # disable type summary for pointers, and make a Python regex summary
        self.runCmd('type summary add i_am_cool -p --summary-string "Text summary"')
        self.runCmd('type summary add -x cool --python-script "%s"' % script)

        # variables should stick to the type summary
        self.expect("frame variable one", substrs=["Text summary"])

        self.expect("frame variable two", substrs=["Text summary"])

        # array and pointer should match the Python one
        self.expect("frame variable twoptr", substrs=["Python summary"])

        self.expect("frame variable array", substrs=["Python summary"])

        # return pointers to the type summary
        self.runCmd('type summary add i_am_cool --summary-string "Text summary"')

        self.expect("frame variable one", substrs=["Text summary"])

        self.expect("frame variable two", substrs=["Text summary"])

        self.expect("frame variable twoptr", substrs=["Text summary"])

        self.expect("frame variable array", substrs=["Python summary"])



import lldb
import binascii
import os.path
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestGDBRemoteClient(GDBRemoteTestBase):
    class gPacketResponder(MockGDBServerResponder):
        registers = [
            "name:rax;bitsize:64;offset:0;encoding:uint;format:hex;set:General Purpose Registers;ehframe:0;dwarf:0;",
            "name:rbx;bitsize:64;offset:8;encoding:uint;format:hex;set:General Purpose Registers;ehframe:3;dwarf:3;",
            "name:rcx;bitsize:64;offset:16;encoding:uint;format:hex;set:General Purpose Registers;ehframe:2;dwarf:2;generic:arg4;",
            "name:rdx;bitsize:64;offset:24;encoding:uint;format:hex;set:General Purpose Registers;ehframe:1;dwarf:1;generic:arg3;",
            "name:rdi;bitsize:64;offset:32;encoding:uint;format:hex;set:General Purpose Registers;ehframe:5;dwarf:5;generic:arg1;",
            "name:rsi;bitsize:64;offset:40;encoding:uint;format:hex;set:General Purpose Registers;ehframe:4;dwarf:4;generic:arg2;",
            "name:rbp;bitsize:64;offset:48;encoding:uint;format:hex;set:General Purpose Registers;ehframe:6;dwarf:6;generic:fp;",
            "name:rsp;bitsize:64;offset:56;encoding:uint;format:hex;set:General Purpose Registers;ehframe:7;dwarf:7;generic:sp;",
        ]

        def qRegisterInfo(self, num):
            try:
                return self.registers[num]
            except IndexError:
                return "E45"

        def readRegisters(self):
            return len(self.registers) * 16 * "0"

        def readRegister(self, register):
            return "0000000000000000"

    def test_connect(self):
        """Test connecting to a remote gdb server"""
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        self.assertPacketLogReceived(["qProcessInfo", "qfThreadInfo"])

    def test_attach_fail(self):
        error_msg = "mock-error-msg"

        class MyResponder(MockGDBServerResponder):
            # Pretend we don't have any process during the initial queries.
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "OK"  # No threads.

            # Then, when we are asked to attach, error out.
            def vAttach(self, pid):
                return "E42;" + binascii.hexlify(error_msg.encode()).decode()

        self.server.responder = MyResponder()

        target = self.dbg.CreateTarget("")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        error = lldb.SBError()
        target.AttachToProcessWithID(lldb.SBListener(), 47, error)
        self.assertEqual(error_msg, error.GetCString())

    def test_launch_fail(self):
        class MyResponder(MockGDBServerResponder):
            # Pretend we don't have any process during the initial queries.
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "OK"  # No threads.

            # Then, when we are asked to attach, error out.
            def A(self, packet):
                return "E47"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        error = lldb.SBError()
        target.Launch(
            lldb.SBListener(), None, None, None, None, None, None, 0, True, error
        )
        self.assertRegex(error.GetCString(), "Cannot launch '.*a': Error 71")

    def test_launch_rich_error(self):
        class MyResponder(MockGDBServerResponder):
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "OK"  # No threads.

            # Then, when we are asked to attach, error out.
            def vRun(self, packet):
                return "Eff;" + seven.hexlify("I'm a teapot")

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        error = lldb.SBError()
        target.Launch(
            lldb.SBListener(), None, None, None, None, None, None, 0, True, error
        )
        self.assertRegex(error.GetCString(), "Cannot launch '.*a': I'm a teapot")

    def test_read_registers_using_g_packets(self):
        """Test reading registers using 'g' packets (default behavior)"""
        self.dbg.HandleCommand(
            "settings set plugin.process.gdb-remote.use-g-packet-for-reading true"
        )
        self.addTearDownHook(
            lambda: self.runCmd(
                "settings set plugin.process.gdb-remote.use-g-packet-for-reading false"
            )
        )
        self.server.responder = self.gPacketResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        # We want to make sure that the process is using the g packet, but it's
        # not required the "connect" should read all registers.  However, it might
        # have...  So we need to wait till we explicitly 'read_registers' to do
        # test.
        # Also, even with the use-g-packet-for-reading lldb will sometimes send p0
        # early on to see if the packet is supported.  So we can't say that there
        # will be NO p packets.
        # But there certainly should be no p packets after the g packet.

        self.read_registers(process)
        received = self.server.responder.packetLog.get_received()
        print(f"\nPACKET LOG:\n{received}\n")
        g_pos = 0
        try:
            g_pos = received.index("g")
        except err:
            self.fail("'g' packet not found after fetching registers")

        try:
            second_g = received.index("g", g_pos + 1)
            self.fail("Found more than one 'g' packet")
        except:
            pass

        # Make sure there aren't any `p` packets after the `g` packet:
        self.assertEqual(
            0,
            len([p for p in received[g_pos:] if p.startswith("p")]),
        )

    def test_read_registers_using_p_packets(self):
        """Test reading registers using 'p' packets"""
        self.dbg.HandleCommand(
            "settings set plugin.process.gdb-remote.use-g-packet-for-reading false"
        )
        self.server.responder = self.gPacketResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.read_registers(process)
        received = self.server.responder.packetLog.get_received()
        self.assertNotIn("g", received)
        self.assertGreater(len([p for p in received if p.startswith("p")]), 0)

    def test_write_registers_using_P_packets(self):
        """Test writing registers using 'P' packets (default behavior)"""
        self.server.responder = self.gPacketResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.write_registers(process)
        received = self.server.responder.packetLog.get_received()
        self.assertEqual(0, len([p for p in received if p.startswith("G")]))
        self.assertGreater(len([p for p in received if p.startswith("P")]), 0)

    def test_write_registers_using_G_packets(self):
        """Test writing registers using 'G' packets"""

        class MyResponder(self.gPacketResponder):
            def readRegister(self, register):
                # empty string means unsupported
                return ""

        self.server.responder = MyResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.write_registers(process)
        received = self.server.responder.packetLog.get_received()
        self.assertEqual(0, len([p for p in received if p.startswith("P")]))
        self.assertGreater(len([p for p in received if p.startswith("G")]), 0)

    def read_registers(self, process):
        self.for_each_gpr(
            process, lambda r: self.assertEqual("0x0000000000000000", r.GetValue())
        )

    def write_registers(self, process):
        self.for_each_gpr(
            process, lambda r: r.SetValueFromCString("0x0000000000000000")
        )

    def for_each_gpr(self, process, operation):
        registers = process.GetThreadAtIndex(0).GetFrameAtIndex(0).GetRegisters()
        self.assertGreater(registers.GetSize(), 0)
        regSet = registers[0]
        numChildren = regSet.GetNumChildren()
        self.assertGreater(numChildren, 0)
        for i in range(numChildren):
            operation(regSet.GetChildAtIndex(i))

    def test_launch_A(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, *args, **kwargs):
                self.started = False
                return super().__init__(*args, **kwargs)

            def qC(self):
                if self.started:
                    return "QCp10.10"
                else:
                    return "E42"

            def qfThreadInfo(self):
                if self.started:
                    return "mp10.10"
                else:
                    return "E42"

            def qsThreadInfo(self):
                return "l"

            def A(self, packet):
                self.started = True
                return "OK"

            def qLaunchSuccess(self):
                if self.started:
                    return "OK"
                return "E42"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        # NB: apparently GDB packets are using "/" on Windows too
        exe_path = self.getBuildArtifact("a").replace(os.path.sep, "/")
        exe_hex = binascii.b2a_hex(exe_path.encode()).decode()
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        target.Launch(
            lldb.SBListener(),
            ["arg1", "arg2", "arg3"],  # argv
            [],  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working_directory
            0,  # launch_flags
            True,  # stop_at_entry
            lldb.SBError(),
        )  # error
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 16)

        self.assertPacketLogReceived(
            [
                "A%d,0,%s,8,1,61726731,8,2,61726732,8,3,61726733"
                % (len(exe_hex), exe_hex),
            ]
        )

    def test_launch_vRun(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, *args, **kwargs):
                self.started = False
                return super().__init__(*args, **kwargs)

            def qC(self):
                if self.started:
                    return "QCp10.10"
                else:
                    return "E42"

            def qfThreadInfo(self):
                if self.started:
                    return "mp10.10"
                else:
                    return "E42"

            def qsThreadInfo(self):
                return "l"

            def vRun(self, packet):
                self.started = True
                return "T13"

            def A(self, packet):
                return "E28"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        # NB: apparently GDB packets are using "/" on Windows too
        exe_path = self.getBuildArtifact("a").replace(os.path.sep, "/")
        exe_hex = binascii.b2a_hex(exe_path.encode()).decode()
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        process = target.Launch(
            lldb.SBListener(),
            ["arg1", "arg2", "arg3"],  # argv
            [],  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working_directory
            0,  # launch_flags
            True,  # stop_at_entry
            lldb.SBError(),
        )  # error
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 16)

        self.assertPacketLogReceived(
            ["vRun;%s;61726731;61726732;61726733" % (exe_hex,)]
        )

    def test_launch_lengthy_vRun(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, *args, **kwargs):
                self.started = False
                return super().__init__(*args, **kwargs)

            def qC(self):
                if self.started:
                    return "QCp10.10"
                else:
                    return "E42"

            def qfThreadInfo(self):
                if self.started:
                    return "mp10.10"
                else:
                    return "E42"

            def qsThreadInfo(self):
                return "l"

            def qEcho(self, num):
                resp = "qEcho:" + str(num)
                if num >= 2:
                    # We have launched our program
                    self.started = True
                    return [resp, "T13"]

                return resp

            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+;qEcho+;"

            def qHostInfo(self):
                return "default_packet_timeout:1;"

            def vRun(self, packet):
                return [self.RESPONSE_NONE]

            def A(self, packet):
                return "E28"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        # NB: apparently GDB packets are using "/" on Windows too
        exe_path = self.getBuildArtifact("a").replace(os.path.sep, "/")
        exe_hex = binascii.b2a_hex(exe_path.encode()).decode()
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        process = target.Launch(
            lldb.SBListener(),
            ["arg1", "arg2", "arg3"],  # argv
            [],  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working_directory
            0,  # launch_flags
            True,  # stop_at_entry
            lldb.SBError(),
        )  # error
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetProcessID(), 16)

        self.assertPacketLogReceived(
            ["vRun;%s;61726731;61726732;61726733" % (exe_hex,)]
        )

    def test_launch_QEnvironment(self):
        class MyResponder(MockGDBServerResponder):
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "E42"

            def vRun(self, packet):
                self.started = True
                return "E28"

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        target.Launch(
            lldb.SBListener(),
            [],  # argv
            [
                "PLAIN=foo",
                "NEEDSENC=frob$",
                "NEEDSENC2=fr*ob",
                "NEEDSENC3=fro}b",
                "NEEDSENC4=f#rob",
                "EQUALS=foo=bar",
            ],  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working_directory
            0,  # launch_flags
            True,  # stop_at_entry
            lldb.SBError(),
        )  # error

        self.assertPacketLogReceived(
            [
                "QEnvironment:EQUALS=foo=bar",
                "QEnvironmentHexEncoded:4e45454453454e433d66726f6224",
                "QEnvironmentHexEncoded:4e45454453454e43323d66722a6f62",
                "QEnvironmentHexEncoded:4e45454453454e43333d66726f7d62",
                "QEnvironmentHexEncoded:4e45454453454e43343d6623726f62",
                "QEnvironment:PLAIN=foo",
            ]
        )

    def test_launch_QEnvironmentHexEncoded_only(self):
        class MyResponder(MockGDBServerResponder):
            def qC(self):
                return "E42"

            def qfThreadInfo(self):
                return "E42"

            def vRun(self, packet):
                self.started = True
                return "E28"

            def QEnvironment(self, packet):
                return ""

        self.server.responder = MyResponder()

        target = self.createTarget("a.yaml")
        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        target.Launch(
            lldb.SBListener(),
            [],  # argv
            [
                "PLAIN=foo",
                "NEEDSENC=frob$",
                "NEEDSENC2=fr*ob",
                "NEEDSENC3=fro}b",
                "NEEDSENC4=f#rob",
                "EQUALS=foo=bar",
            ],  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working_directory
            0,  # launch_flags
            True,  # stop_at_entry
            lldb.SBError(),
        )  # error

        self.assertPacketLogReceived(
            [
                "QEnvironmentHexEncoded:455155414c533d666f6f3d626172",
                "QEnvironmentHexEncoded:4e45454453454e433d66726f6224",
                "QEnvironmentHexEncoded:4e45454453454e43323d66722a6f62",
                "QEnvironmentHexEncoded:4e45454453454e43333d66726f7d62",
                "QEnvironmentHexEncoded:4e45454453454e43343d6623726f62",
                "QEnvironmentHexEncoded:504c41494e3d666f6f",
            ]
        )

    def test_detach_no_multiprocess(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self):
                super().__init__()
                self.detached = None

            def qfThreadInfo(self):
                return "10200"

            def D(self, packet):
                self.detached = packet
                return "OK"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        process = self.connect(target)
        process.Detach()
        self.assertEqual(self.server.responder.detached, "D")

    def test_detach_pid(self):
        class MyResponder(MockGDBServerResponder):
            def __init__(self, test_case):
                super().__init__()
                self.test_case = test_case
                self.detached = None

            def qSupported(self, client_supported):
                self.test_case.assertIn("multiprocess+", client_supported)
                return "multiprocess+;" + super().qSupported(client_supported)

            def qfThreadInfo(self):
                return "mp400.10200"

            def D(self, packet):
                self.detached = packet
                return "OK"

        self.server.responder = MyResponder(self)
        target = self.dbg.CreateTarget("")
        process = self.connect(target)
        process.Detach()
        self.assertRegex(self.server.responder.detached, r"D;0*400")

    def test_signal_gdb(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+"

            def haltReason(self):
                return "S0a"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.threads[0].GetStopReason(), lldb.eStopReasonSignal)
        self.assertEqual(process.threads[0].stop_description, "signal SIGBUS")

    def test_signal_lldb_old(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+"

            def qHostInfo(self):
                return "triple:61726d76372d756e6b6e6f776e2d6c696e75782d676e75;"

            def QThreadSuffixSupported(self):
                return "OK"

            def haltReason(self):
                return "S0a"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.threads[0].GetStopReason(), lldb.eStopReasonSignal)
        self.assertEqual(process.threads[0].stop_description, "signal SIGUSR1")

    def test_signal_lldb(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+;native-signals+"

            def qHostInfo(self):
                return "triple:61726d76372d756e6b6e6f776e2d6c696e75782d676e75;"

            def haltReason(self):
                return "S0a"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select remote-linux")
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        self.assertEqual(process.threads[0].GetStopReason(), lldb.eStopReasonSignal)
        self.assertEqual(process.threads[0].stop_description, "signal SIGUSR1")

    def do_siginfo_test(self, platform, target_yaml, raw_data, expected):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_supported):
                return "PacketSize=3fff;QStartNoAckMode+;qXfer:siginfo:read+"

            def qXferRead(self, obj, annex, offset, length):
                if obj == "siginfo":
                    return raw_data, False
                else:
                    return None, False

            def haltReason(self):
                return "T02"

            def cont(self):
                return self.haltReason()

        self.server.responder = MyResponder()

        self.runCmd("platform select " + platform)
        target = self.createTarget(target_yaml)
        process = self.connect(target)

        siginfo = process.threads[0].GetSiginfo()
        self.assertSuccess(siginfo.GetError())

        for key, value in expected.items():
            self.assertEqual(
                siginfo.GetValueForExpressionPath("." + key).GetValueAsUnsigned(), value
            )

    def test_siginfo_linux_amd64(self):
        data = (
            # si_signo         si_errno        si_code
            "\x11\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00"
            # __pad0           si_pid          si_uid
            "\x00\x00\x00\x00\xbf\xf7\x0b\x00\xe8\x03\x00\x00"
            # si_status
            "\x0c\x00\x00\x00"
            + "\x00" * 100
        )
        expected = {
            "si_signo": 17,  # SIGCHLD
            "si_errno": 0,
            "si_code": 1,  # CLD_EXITED
            "_sifields._sigchld.si_pid": 784319,
            "_sifields._sigchld.si_uid": 1000,
            "_sifields._sigchld.si_status": 12,
            "_sifields._sigchld.si_utime": 0,
            "_sifields._sigchld.si_stime": 0,
        }
        self.do_siginfo_test("remote-linux", "basic_eh_frame.yaml", data, expected)

    def test_siginfo_linux_i386(self):
        data = (
            # si_signo         si_errno        si_code
            "\x11\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00"
            # si_pid           si_uid          si_status
            "\x49\x43\x07\x00\xe8\x03\x00\x00\x0c\x00\x00\x00"
            + "\x00" * 104
        )
        expected = {
            "si_signo": 17,  # SIGCHLD
            "si_errno": 0,
            "si_code": 1,  # CLD_EXITED
            "_sifields._sigchld.si_pid": 475977,
            "_sifields._sigchld.si_uid": 1000,
            "_sifields._sigchld.si_status": 12,
            "_sifields._sigchld.si_utime": 0,
            "_sifields._sigchld.si_stime": 0,
        }
        self.do_siginfo_test("remote-linux", "basic_eh_frame-i386.yaml", data, expected)

    def test_siginfo_freebsd_amd64(self):
        data = (
            # si_signo         si_errno        si_code
            "\x0b\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00"
            # si_pid           si_uid          si_status
            "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
            # si_addr
            "\x76\x98\xba\xdc\xfe\x00\x00\x00"
            # si_status                        si_trapno
            "\x00\x00\x00\x00\x00\x00\x00\x00\x0c\x00\x00\x00"
            + "\x00" * 36
        )

        expected = {
            "si_signo": 11,  # SIGSEGV
            "si_errno": 0,
            "si_code": 1,  # SEGV_MAPERR
            "si_addr": 0xFEDCBA9876,
            "_reason._fault._trapno": 12,
        }
        self.do_siginfo_test("remote-freebsd", "basic_eh_frame.yaml", data, expected)



"""
Test the "process continue --reverse" and "--forward" options
when reverse-continue is not supported.
"""


import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestReverseContinueCommandNotSupported(TestBase):
    def test_reverse_continue_not_supported(self):
        target = self.connect()

        # Set breakpoint and reverse-continue
        trigger_bkpt = target.BreakpointCreateByName("trigger_breakpoint", None)
        self.assertTrue(trigger_bkpt, VALID_BREAKPOINT)
        # `process continue --forward` should work.
        self.expect(
            "process continue --forward",
            substrs=["stop reason = breakpoint {0}.1".format(trigger_bkpt.GetID())],
        )
        self.expect(
            "process continue --reverse",
            error=True,
            # This error is "<plugin name> does not support...". The plugin name changes
            # between platforms.
            substrs=["does not support reverse execution of processes"],
        )

    def test_reverse_continue_forward_and_reverse(self):
        self.connect()

        self.expect(
            "process continue --forward --reverse",
            error=True,
            substrs=["invalid combination of options for the given command"],
        )

    def connect(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_bkpt = target.BreakpointCreateByName("main", None)
        self.assertTrue(main_bkpt, VALID_BREAKPOINT)

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)
        return target



#!/usr/bin/env python

import lldb
import struct


class OperatingSystemPlugIn(object):
    """Class that provides data for an instance of a LLDB 'OperatingSystemPython' plug-in class"""

    def __init__(self, process):
        """Initialization needs a valid.SBProcess object.

        This plug-in will get created after a live process is valid and has stopped for the
        first time."""
        self.process = None
        self.registers = None
        self.threads = None
        if isinstance(process, lldb.SBProcess) and process.IsValid():
            self.process = process
            self.threads = None  # Will be an dictionary containing info for each thread

    def get_target(self):
        # NOTE: Don't use "lldb.target" when trying to get your target as the "lldb.target"
        # tracks the current target in the LLDB command interpreter which isn't the
        # correct thing to use for this plug-in.
        return self.process.target

    def create_thread(self, tid, context):
        if tid == 0x444444444:
            thread_info = {
                "tid": tid,
                "name": "four",
                "queue": "queue4",
                "state": "stopped",
                "stop_reason": "none",
            }
            self.threads.append(thread_info)
            return thread_info
        return None

    def get_thread_info(self):
        if not self.threads:
            # The sample dictionary below shows the values that can be returned for a thread
            # tid => thread ID (mandatory)
            # name => thread name (optional key/value pair)
            # queue => thread dispatch queue name (optional key/value pair)
            # state => thred state (mandatory, set to 'stopped' for now)
            # stop_reason => thread stop reason. (mandatory, usually set to 'none')
            #  Possible values include:
            #   'breakpoint' if the thread is stopped at a breakpoint
            #   'none' thread is just stopped because the process is stopped
            #   'trace' the thread just single stepped
            #   The usual value for this while threads are in memory is 'none'
            # register_data_addr => the address of the register data in memory (optional key/value pair)
            #   Specifying this key/value pair for a thread will avoid a call to get_register_data()
            #   and can be used when your registers are in a thread context structure that is contiguous
            #   in memory. Don't specify this if your register layout in memory doesn't match the layout
            # described by the dictionary returned from a call to the
            # get_register_info() method.
            self.threads = [
                {
                    "tid": 0x111111111,
                    "name": "one",
                    "queue": "queue1",
                    "state": "stopped",
                    "stop_reason": "breakpoint",
                },
                {
                    "tid": 0x222222222,
                    "name": "two",
                    "queue": "queue2",
                    "state": "stopped",
                    "stop_reason": "none",
                },
                {
                    "tid": 0x333333333,
                    "name": "three",
                    "queue": "queue3",
                    "state": "stopped",
                    "stop_reason": "trace",
                    "register_data_addr": 0x100000000,
                },
            ]
        return self.threads

    def get_register_info(self):
        if self.registers is None:
            self.registers = dict()
            triple = self.process.target.triple
            if triple:
                arch = triple.split("-")[0]
                if arch == "x86_64":
                    self.registers["sets"] = ["GPR", "FPU", "EXC"]
                    self.registers["registers"] = [
                        {
                            "name": "rax",
                            "bitsize": 64,
                            "offset": 0,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 0,
                            "dwarf": 0,
                        },
                        {
                            "name": "rbx",
                            "bitsize": 64,
                            "offset": 8,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 3,
                            "dwarf": 3,
                        },
                        {
                            "name": "rcx",
                            "bitsize": 64,
                            "offset": 16,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 2,
                            "dwarf": 2,
                            "generic": "arg4",
                            "alt-name": "arg4",
                        },
                        {
                            "name": "rdx",
                            "bitsize": 64,
                            "offset": 24,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 1,
                            "dwarf": 1,
                            "generic": "arg3",
                            "alt-name": "arg3",
                        },
                        {
                            "name": "rdi",
                            "bitsize": 64,
                            "offset": 32,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 5,
                            "dwarf": 5,
                            "generic": "arg1",
                            "alt-name": "arg1",
                        },
                        {
                            "name": "rsi",
                            "bitsize": 64,
                            "offset": 40,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 4,
                            "dwarf": 4,
                            "generic": "arg2",
                            "alt-name": "arg2",
                        },
                        {
                            "name": "rbp",
                            "bitsize": 64,
                            "offset": 48,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 6,
                            "dwarf": 6,
                            "generic": "fp",
                            "alt-name": "fp",
                        },
                        {
                            "name": "rsp",
                            "bitsize": 64,
                            "offset": 56,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 7,
                            "dwarf": 7,
                            "generic": "sp",
                            "alt-name": "sp",
                        },
                        {
                            "name": "r8",
                            "bitsize": 64,
                            "offset": 64,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 8,
                            "dwarf": 8,
                            "generic": "arg5",
                            "alt-name": "arg5",
                        },
                        {
                            "name": "r9",
                            "bitsize": 64,
                            "offset": 72,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 9,
                            "dwarf": 9,
                            "generic": "arg6",
                            "alt-name": "arg6",
                        },
                        {
                            "name": "r10",
                            "bitsize": 64,
                            "offset": 80,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 10,
                            "dwarf": 10,
                        },
                        {
                            "name": "r11",
                            "bitsize": 64,
                            "offset": 88,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 11,
                            "dwarf": 11,
                        },
                        {
                            "name": "r12",
                            "bitsize": 64,
                            "offset": 96,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 12,
                            "dwarf": 12,
                        },
                        {
                            "name": "r13",
                            "bitsize": 64,
                            "offset": 104,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 13,
                            "dwarf": 13,
                        },
                        {
                            "name": "r14",
                            "bitsize": 64,
                            "offset": 112,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 14,
                            "dwarf": 14,
                        },
                        {
                            "name": "r15",
                            "bitsize": 64,
                            "offset": 120,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 15,
                            "dwarf": 15,
                        },
                        {
                            "name": "rip",
                            "bitsize": 64,
                            "offset": 128,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "gcc": 16,
                            "dwarf": 16,
                            "generic": "pc",
                            "alt-name": "pc",
                        },
                        {
                            "name": "rflags",
                            "bitsize": 64,
                            "offset": 136,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                            "generic": "flags",
                            "alt-name": "flags",
                        },
                        {
                            "name": "cs",
                            "bitsize": 64,
                            "offset": 144,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                        },
                        {
                            "name": "fs",
                            "bitsize": 64,
                            "offset": 152,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                        },
                        {
                            "name": "gs",
                            "bitsize": 64,
                            "offset": 160,
                            "encoding": "uint",
                            "format": "hex",
                            "set": 0,
                        },
                    ]
        return self.registers

    def get_register_data(self, tid):
        if tid == 0x111111111:
            return struct.pack(
                "21Q",
                1,
                2,
                3,
                4,
                5,
                6,
                7,
                8,
                9,
                10,
                11,
                12,
                13,
                14,
                15,
                16,
                17,
                18,
                19,
                20,
                21,
            )
        elif tid == 0x222222222:
            return struct.pack(
                "21Q",
                11,
                12,
                13,
                14,
                15,
                16,
                17,
                18,
                19,
                110,
                111,
                112,
                113,
                114,
                115,
                116,
                117,
                118,
                119,
                120,
                121,
            )
        elif tid == 0x333333333:
            return struct.pack(
                "21Q",
                21,
                22,
                23,
                24,
                25,
                26,
                27,
                28,
                29,
                210,
                211,
                212,
                213,
                214,
                215,
                216,
                217,
                218,
                219,
                220,
                221,
            )
        elif tid == 0x444444444:
            return struct.pack(
                "21Q",
                31,
                32,
                33,
                34,
                35,
                36,
                37,
                38,
                39,
                310,
                311,
                312,
                313,
                314,
                315,
                316,
                317,
                318,
                319,
                320,
                321,
            )
        else:
            return struct.pack(
                "21Q",
                41,
                42,
                43,
                44,
                45,
                46,
                47,
                48,
                49,
                410,
                411,
                412,
                413,
                414,
                415,
                416,
                417,
                418,
                419,
                420,
                421,
            )
        return None



import lldb
from lldbsuite.test.lldbtest import TestBase
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "return", lldb.SBFileSpec("main.c"))

        # Expect "frame #0" but not "frame #1".
        self.expect("bt 1", inHistory=True, patterns=["frame #0", "(?!.*frame #1)"])

        # Run an empty command to run the repeat command for `bt`.
        # The repeat command for `bt N` lists the subsequent N frames.
        #
        # In this case, after printing the frame 0 with `bt 1`, the repeat
        # command will print "frame #1" (and won't print "frame #0").
        self.expect("", patterns=["^(?!.*frame #0)", "frame #1"])



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StdVariantDataFormatterTestCase(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def do_test(self):
        """Test that that file and class static variables display correctly."""

        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        (self.target, self.process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        for name in ["v1", "v1_typedef"]:
            self.expect(
                "frame variable " + name,
                substrs=[name + " =  Active Type = int  {", "Value = 12", "}"],
            )

        for name in ["v1_ref", "v1_typedef_ref"]:
            self.expect(
                "frame variable " + name,
                patterns=[name + " = 0x.*  Active Type = int : {", "Value = 12", "}"],
            )

        self.expect(
            "frame variable v_v1",
            substrs=[
                "v_v1 =  Active Type = std::variant<int, double, char>  {",
                "Value =  Active Type = int  {",
                "Value = 12",
                "}",
                "}",
            ],
        )

        self.expect_expr(
            "v4",
            result_summary=" Active Type = int ",
            result_children=[
                ValueCheck(name="Value", value="4"),
            ],
        )

        lldbutil.continue_to_breakpoint(self.process, bkpt)

        self.expect(
            "frame variable v1",
            substrs=["v1 =  Active Type = double  {", "Value = 2", "}"],
        )

        lldbutil.continue_to_breakpoint(self.process, bkpt)

        self.expect(
            "frame variable v2",
            substrs=["v2 =  Active Type = double  {", "Value = 2", "}"],
        )

        self.expect(
            "frame variable v3",
            substrs=["v3 =  Active Type = char  {", "Value = 'A'", "}"],
        )

        if self.getDebugInfo() == "pdb":
            string_name = (
                "std::basic_string<char, std::char_traits<char>, std::allocator<char>>"
            )
        elif self.platformIsDarwin():
            string_name = "std::string"
        else:
            string_name = "std::basic_string<char>"

        self.expect_expr(
            "v4",
            result_summary=f" Active Type = {string_name} ",
            result_children=[
                ValueCheck(name="Value", summary='"a string"'),
            ],
        )

        self.expect("frame variable v_valueless", substrs=["v_valueless =  No Value"])

        self.expect(
            "frame variable v_300_types_valueless",
            substrs=["v_300_types_valueless =  No Value"],
        )

    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()

    @add_test_categories(["libstdcxx"])
    def test_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @add_test_categories(["msvcstl"])
    def test_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test()



##===-- debuggerdriver.py ------------------------------------*- Python -*-===##
##
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
##
##===----------------------------------------------------------------------===##


import lldb
import lldbutil
import sys
from threading import Thread


class DebuggerDriver(Thread):
    """Drives the debugger and responds to events."""

    def __init__(self, debugger, event_queue):
        Thread.__init__(self)
        self.event_queue = event_queue
        # This is probably not great because it does not give liblldb a chance
        # to clean up
        self.daemon = True
        self.initialize(debugger)

    def initialize(self, debugger):
        self.done = False
        self.debugger = debugger
        self.listener = debugger.GetListener()
        if not self.listener.IsValid():
            raise "Invalid listener"

        self.listener.StartListeningForEventClass(
            self.debugger,
            lldb.SBTarget.GetBroadcasterClassName(),
            lldb.SBTarget.eBroadcastBitBreakpointChanged
            # | lldb.SBTarget.eBroadcastBitModuleLoaded
            # | lldb.SBTarget.eBroadcastBitModuleUnloaded
            | lldb.SBTarget.eBroadcastBitWatchpointChanged
            # | lldb.SBTarget.eBroadcastBitSymbolLoaded
        )

        self.listener.StartListeningForEventClass(
            self.debugger,
            lldb.SBThread.GetBroadcasterClassName(),
            lldb.SBThread.eBroadcastBitStackChanged
            #  lldb.SBThread.eBroadcastBitBreakpointChanged
            | lldb.SBThread.eBroadcastBitThreadSuspended
            | lldb.SBThread.eBroadcastBitThreadResumed
            | lldb.SBThread.eBroadcastBitSelectedFrameChanged
            | lldb.SBThread.eBroadcastBitThreadSelected,
        )

        self.listener.StartListeningForEventClass(
            self.debugger,
            lldb.SBProcess.GetBroadcasterClassName(),
            lldb.SBProcess.eBroadcastBitStateChanged
            | lldb.SBProcess.eBroadcastBitInterrupt
            | lldb.SBProcess.eBroadcastBitSTDOUT
            | lldb.SBProcess.eBroadcastBitSTDERR
            | lldb.SBProcess.eBroadcastBitProfileData,
        )
        self.listener.StartListeningForEventClass(
            self.debugger,
            lldb.SBCommandInterpreter.GetBroadcasterClass(),
            lldb.SBCommandInterpreter.eBroadcastBitThreadShouldExit
            | lldb.SBCommandInterpreter.eBroadcastBitResetPrompt
            | lldb.SBCommandInterpreter.eBroadcastBitQuitCommandReceived
            | lldb.SBCommandInterpreter.eBroadcastBitAsynchronousOutputData
            | lldb.SBCommandInterpreter.eBroadcastBitAsynchronousErrorData,
        )

    def createTarget(self, target_image, args=None):
        self.handleCommand("target create %s" % target_image)
        if args is not None:
            self.handleCommand("settings set target.run-args %s" % args)

    def attachProcess(self, pid):
        self.handleCommand("process attach -p %d" % pid)
        pass

    def loadCore(self, corefile):
        self.handleCommand("target create -c %s" % corefile)
        pass

    def setDone(self):
        self.done = True

    def isDone(self):
        return self.done

    def getPrompt(self):
        return self.debugger.GetPrompt()

    def getCommandInterpreter(self):
        return self.debugger.GetCommandInterpreter()

    def getSourceManager(self):
        return self.debugger.GetSourceManager()

    def setSize(self, width, height):
        # FIXME: respect height
        self.debugger.SetTerminalWidth(width)

    def getTarget(self):
        return self.debugger.GetTargetAtIndex(0)

    def handleCommand(self, cmd):
        ret = lldb.SBCommandReturnObject()
        self.getCommandInterpreter().HandleCommand(cmd, ret)
        return ret

    def eventLoop(self):
        while not self.isDone():
            event = lldb.SBEvent()
            got_event = self.listener.WaitForEvent(lldb.UINT32_MAX, event)
            if got_event and not event.IsValid():
                self.winAddStr("Warning: Invalid or no event...")
                continue
            elif not event.GetBroadcaster().IsValid():
                continue

            self.event_queue.put(event)

    def run(self):
        self.eventLoop()

    def terminate(self):
        lldb.SBDebugger.Terminate()
        sys.exit(0)


def createDriver(debugger, event_queue):
    driver = DebuggerDriver(debugger, event_queue)
    # driver.start()
    # if pid specified:
    # - attach to pid
    # else if core file specified
    # - create target from corefile
    # else
    # - create target from file
    # - settings append target.run-args <args-from-cmdline>
    # source .lldbinit file

    return driver



import shlex


class BuildError(Exception):
    def __init__(self, called_process_error):
        super(BuildError, self).__init__("Error when building test subject")
        self.command = shlex.join(called_process_error.cmd)
        self.build_error = called_process_error.output

    def __str__(self):
        return self.format_build_error(self.command, self.build_error)

    @staticmethod
    def format_build_error(command, command_output):
        return "Error when building test subject.\n\nBuild Command:\n{}\n\nBuild Command Output:\n{}".format(
            command, command_output
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import nvgpu


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        print(module)
    return f


@run
def testCreateAsyncGroups():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate, [], transform.AnyOpType.get()
    )
    with InsertionPoint(sequence.body):
        nvgpu.CreateAsyncGroupsOp(transform.AnyOpType.get(), sequence.bodyTarget)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testCreateAsyncGroups
    # CHECK: transform.nvgpu.create_async_groups



"""Test stepping through ObjC method dispatch in various forms."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCStepping(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "stepping-tests.m"
        self.source_randomMethod_line = line_number(
            self.main_source, "// Source randomMethod start line."
        )
        self.sourceBase_randomMethod_line = line_number(
            self.main_source, "// SourceBase randomMethod start line."
        )
        self.source_returnsStruct_start_line = line_number(
            self.main_source, "// Source returnsStruct start line."
        )
        self.sourceBase_returnsStruct_start_line = line_number(
            self.main_source, "// SourceBase returnsStruct start line."
        )
        self.stepped_past_nil_line = line_number(
            self.main_source, "// Step over nil should stop here."
        )

    @add_test_categories(["pyapi", "basic_process"])
    def test_with_python_api(self):
        """Test stepping through ObjC method dispatch in various forms."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        self.main_source_spec = lldb.SBFileSpec(self.main_source)

        breakpoints_to_disable = []

        break1 = target.BreakpointCreateBySourceRegex(
            "// Set first breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break1, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break1)

        break2 = target.BreakpointCreateBySourceRegex(
            "// Set second breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break2, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break2)

        break3 = target.BreakpointCreateBySourceRegex(
            "// Set third breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break3, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break3)

        break4 = target.BreakpointCreateBySourceRegex(
            "// Set fourth breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break4, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break4)

        break5 = target.BreakpointCreateBySourceRegex(
            "// Set fifth breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break5, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break5)

        break_returnStruct_call_super = target.BreakpointCreateBySourceRegex(
            "// Source returnsStruct call line.", self.main_source_spec
        )
        self.assertTrue(break_returnStruct_call_super, VALID_BREAKPOINT)
        breakpoints_to_disable.append(break_returnStruct_call_super)

        break_step_nil = target.BreakpointCreateBySourceRegex(
            "// Set nil step breakpoint here.", self.main_source_spec
        )
        self.assertTrue(break_step_nil, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break1)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint 1.")

        thread = threads[0]

        mySource = thread.GetFrameAtIndex(0).FindVariable("mySource")
        self.assertTrue(mySource, "Found mySource local variable.")
        mySource_isa = mySource.GetChildMemberWithName("isa")
        self.assertTrue(mySource_isa, "Found mySource->isa local variable.")
        className = mySource_isa.GetSummary()

        if self.TraceOn():
            print(mySource_isa)

        # Lets delete mySource so we can check that after stepping a child variable
        # with no parent persists and is useful.
        del mySource

        # Now step in, that should leave us in the Source randomMethod:
        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.source_randomMethod_line,
            "Stepped into Source randomMethod.",
        )

        # Now step in again, through the super call, and that should leave us
        # in the SourceBase randomMethod:
        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.sourceBase_randomMethod_line,
            "Stepped through super into SourceBase randomMethod.",
        )

        threads = lldbutil.continue_to_breakpoint(process, break2)
        self.assertEqual(len(threads), 1, "Continued to second breakpoint in main.")

        # Again, step in twice gets us to a stret method and a stret super
        # call:
        thread = threads[0]
        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.source_returnsStruct_start_line,
            "Stepped into Source returnsStruct.",
        )

        threads = lldbutil.continue_to_breakpoint(
            process, break_returnStruct_call_super
        )
        self.assertEqual(
            len(threads), 1, "Stepped to the call super line in Source returnsStruct."
        )
        thread = threads[0]

        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.sourceBase_returnsStruct_start_line,
            "Stepped through super into SourceBase returnsStruct.",
        )

        # Cool now continue to get past the call that initializes the Observer, and then do our steps in again to see that
        # we can find our way when we're stepping through a KVO swizzled
        # object.

        threads = lldbutil.continue_to_breakpoint(process, break3)
        self.assertEqual(
            len(threads),
            1,
            "Continued to third breakpoint in main, our object should now be swizzled.",
        )

        newClassName = mySource_isa.GetSummary()

        if self.TraceOn():
            print("className is %s, newClassName is %s" % (className, newClassName))
            print(mySource_isa)

        self.assertNotEqual(
            newClassName, className, "The isa did indeed change, swizzled!"
        )

        # Now step in, that should leave us in the Source randomMethod:
        thread = threads[0]
        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.source_randomMethod_line,
            "Stepped into Source randomMethod in swizzled object.",
        )

        # Now step in again, through the super call, and that should leave us
        # in the SourceBase randomMethod:
        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.sourceBase_randomMethod_line,
            "Stepped through super into SourceBase randomMethod in swizzled object.",
        )

        threads = lldbutil.continue_to_breakpoint(process, break4)
        self.assertEqual(len(threads), 1, "Continued to fourth breakpoint in main.")
        thread = threads[0]

        # Again, step in twice gets us to a stret method and a stret super
        # call:
        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.source_returnsStruct_start_line,
            "Stepped into Source returnsStruct in swizzled object.",
        )

        threads = lldbutil.continue_to_breakpoint(
            process, break_returnStruct_call_super
        )
        self.assertEqual(
            len(threads),
            1,
            "Stepped to the call super line in Source returnsStruct - second time.",
        )
        thread = threads[0]

        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.sourceBase_returnsStruct_start_line,
            "Stepped through super into SourceBase returnsStruct in swizzled object.",
        )

        for bkpt in breakpoints_to_disable:
            bkpt.SetEnabled(False)

        threads = lldbutil.continue_to_breakpoint(process, break_step_nil)
        self.assertEqual(len(threads), 1, "Continued to step nil breakpoint.")

        thread.StepInto()
        line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()
        self.assertEqual(
            line_number,
            self.stepped_past_nil_line,
            "Step in over dispatch to nil stepped over.",
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestNamespaceLocalVarSameNameCppAndC(TestBase):
    @skipUnlessDarwin
    @add_test_categories(["gmodules"])
    def test_namespace_local_var_same_name_cpp_and_c(self):
        self.build()

        (self.target, self.process, _, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect_expr("error", result_type="int", result_value="1")
        lldbutil.continue_to_breakpoint(self.process, bkpt)
        self.expect_expr("error", result_type="int", result_value="1")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestWithLimitDebugInfo(TestBase):
    SHARED_BUILD_TESTCASE = False

    def _run_test(self, build_dict):
        self.build(dictionary=build_dict)

        # Get the path of the executable
        exe_path = self.getBuildArtifact("a.out")

        # Load the executable
        target = self.dbg.CreateTarget(exe_path)
        self.assertTrue(target.IsValid(), VALID_TARGET)

        # Break on main function
        lldbutil.run_break_set_by_file_and_line(
            self, "derived.h", line_number("derived.h", "// break1")
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "derived.h", line_number("derived.h", "// break2")
        )

        # Launch the process
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process.IsValid(), PROCESS_IS_VALID)

        # Get the thread of the process
        self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        self.expect_expr("1", result_type="int", result_value="1")
        self.expect_expr("this", result_type="Foo *")
        self.expect_expr("this->x", result_type="int", result_value="12345")

        self.runCmd("continue")

        self.expect_expr("1", result_type="int", result_value="1")
        self.expect_expr("this", result_type="ns::Foo2 *")
        self.expect_expr("this->x", result_type="int", result_value="23456")

    @add_test_categories(["dwarf", "dwo"])
    def test_default(self):
        self._run_test(dict(CFLAGS_EXTRAS="$(LIMIT_DEBUG_INFO_FLAGS)"))

    @add_test_categories(["dwarf", "dwo"])
    def test_debug_names(self):
        self._run_test(
            dict(CFLAGS_EXTRAS="$(LIMIT_DEBUG_INFO_FLAGS) -gdwarf-5 -gpubnames")
        )



import lldb
from lldbsuite.support import seven
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestReadMemory(GDBRemoteTestBase):
    def test_x_with_prefix(self):
        class MyResponder(MockGDBServerResponder):
            def qSupported(self, client_features):
                # binary-upload+ indicates we use the gdb style of x packets
                return super().qSupported(client_features) + ";binary-upload+"

            def x(self, addr, length):
                return "bfoobar" if addr == 0x1000 else "E01"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTargetWithFileAndTargetTriple("", "x86_64-pc-linux")
        process = self.connect(target)

        error = lldb.SBError()
        self.assertEqual(b"foobar", process.ReadMemory(0x1000, 10, error))

    def test_x_bare(self):
        class MyResponder(MockGDBServerResponder):
            def x(self, addr, length):
                # The OK response indicates we use the old lldb style.
                if addr == 0 and length == 0:
                    return "OK"
                return "foobar" if addr == 0x1000 else "E01"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTargetWithFileAndTargetTriple("", "x86_64-pc-linux")
        process = self.connect(target)

        error = lldb.SBError()
        self.assertEqual(b"foobar", process.ReadMemory(0x1000, 10, error))

    def test_m_fallback(self):
        class MyResponder(MockGDBServerResponder):
            def x(self, addr, length):
                # If `x` is unsupported, we should fall back to `m`.
                return ""

            def readMemory(self, addr, length):
                return seven.hexlify("foobar") if addr == 0x1000 else "E01"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTargetWithFileAndTargetTriple("", "x86_64-pc-linux")
        process = self.connect(target)

        error = lldb.SBError()
        self.assertEqual(b"foobar", process.ReadMemory(0x1000, 10, error))



"""
Confirm that lldb modify watchpoints only stop
when the value being watched changes.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipIfWindows
class ModifyWatchpointTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_modify_watchpoint(self):
        """Test that a modify watchpoint only stops when the value changes."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", self.main_source_file
        )

        self.runCmd("watch set variable value")
        process.Continue()
        frame = process.GetSelectedThread().GetFrameAtIndex(0)
        self.assertEqual(frame.locals["value"][0].GetValueAsUnsigned(), 10)

        process.Continue()
        frame = process.GetSelectedThread().GetFrameAtIndex(0)
        self.assertEqual(frame.locals["value"][0].GetValueAsUnsigned(), 5)

        process.Continue()
        frame = process.GetSelectedThread().GetFrameAtIndex(0)
        self.assertEqual(frame.locals["value"][0].GetValueAsUnsigned(), 7)

        process.Continue()
        frame = process.GetSelectedThread().GetFrameAtIndex(0)
        self.assertEqual(frame.locals["value"][0].GetValueAsUnsigned(), 9)



"""
Test that argdumper is a viable launching strategy.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestRerun(TestBase):
    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        self.runCmd("target create %s" % exe)

        # Create the target
        target = self.dbg.CreateTarget(exe)

        # Create any breakpoints we need
        breakpoint = target.BreakpointCreateBySourceRegex(
            "break here", lldb.SBFileSpec("main.cpp", False)
        )
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        self.runCmd("process launch 1 2 3")

        process = self.process()
        thread = lldbutil.get_one_thread_stopped_at_breakpoint(process, breakpoint)
        self.assertIsNotNone(
            thread, "Process should be stopped at a breakpoint in main"
        )
        self.assertTrue(thread.IsValid(), "Stopped thread is not valid")

        self.expect("frame variable argv[1]", substrs=["1"])
        self.expect("frame variable argv[2]", substrs=["2"])
        self.expect("frame variable argv[3]", substrs=["3"])

        # Let program exit
        self.runCmd("continue")

        # Re-run with no args and make sure we still run with 1 2 3 as arguments as
        # they should have been stored in "target.run-args"
        self.runCmd("process launch")

        process = self.process()
        thread = lldbutil.get_one_thread_stopped_at_breakpoint(process, breakpoint)

        self.assertIsNotNone(
            thread, "Process should be stopped at a breakpoint in main"
        )
        self.assertTrue(thread.IsValid(), "Stopped thread is not valid")

        self.expect("frame variable argv[1]", substrs=["1"])
        self.expect("frame variable argv[2]", substrs=["2"])
        self.expect("frame variable argv[3]", substrs=["3"])



"""
Test that global operators are found and evaluated.
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCppGlobalOperators(TestBase):
    def prepare_executable_and_get_frame(self):
        self.build()

        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        return thread.GetSelectedFrame()

    def test_equals_operator(self):
        frame = self.prepare_executable_and_get_frame()

        test_result = frame.EvaluateExpression("operator==(s1, s2)")
        self.assertTrue(
            test_result.IsValid() and test_result.GetValue() == "false",
            "operator==(s1, s2) = false",
        )

        test_result = frame.EvaluateExpression("operator==(s1, s3)")
        self.assertTrue(
            test_result.IsValid() and test_result.GetValue() == "true",
            "operator==(s1, s3) = true",
        )

        test_result = frame.EvaluateExpression("operator==(s2, s3)")
        self.assertTrue(
            test_result.IsValid() and test_result.GetValue() == "false",
            "operator==(s2, s3) = false",
        )

    def do_new_test(self, frame, expr, expected_value_name):
        """Evaluate a new expression, and check its result"""

        expected_value = frame.FindValue(
            expected_value_name, lldb.eValueTypeVariableGlobal
        )
        self.assertTrue(expected_value.IsValid())

        expected_value_addr = expected_value.AddressOf()
        self.assertTrue(expected_value_addr.IsValid())

        got = frame.EvaluateExpression(expr)
        self.assertTrue(got.IsValid())
        self.assertEqual(
            got.GetValueAsUnsigned(), expected_value_addr.GetValueAsUnsigned()
        )
        got_type = got.GetType()
        self.assertTrue(got_type.IsPointerType())
        self.assertEqual(got_type.GetPointeeType().GetName(), "Struct")

    @skipIfMTE  # Expression evaluation of overridden operator new fails under MTE.
    def test_operator_new(self):
        frame = self.prepare_executable_and_get_frame()

        self.do_new_test(frame, "new Struct()", "global_new_buf")
        self.do_new_test(frame, "new(new_tag) Struct()", "tagged_new_buf")



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CompactVectorsFormattingTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    @skipIf(compiler="clang", compiler_version=["<", "19.0"])
    @skipUnlessDarwin
    def test_with_run_command(self):
        """Test that that file and class static variables display correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.expect(
            "frame variable",
            substrs=[
                "(vFloat) valueFL = (1.25, 0, 0.25, 0)",
                "(vDouble) valueDL = (1.25, 2.25)",
                "(int16_t[8]) valueI16 = (1, 0, 4, 0, 0, 1, 0, 4)",
                "(int32_t[4]) valueI32 = (1, 0, 4, 0)",
                "(vUInt8) valueU8 = (0x01, 0x00, 0x04, 0x00, 0x00, 0x01, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)",
                "(vUInt16) valueU16 = (1, 0, 4, 0, 0, 1, 0, 4)",
                "(vUInt32) valueU32 = (1, 2, 3, 4)",
                "(vSInt8) valueS8 = (1, 0, 4, 0, 0, 1, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0)",
                "(vSInt16) valueS16 = (1, 0, 4, 0, 0, 1, 0, 4)",
                "(vSInt32) valueS32 = (4, 3, 2, 1)",
                "(vBool32) valueBool32 = (0, 1, 0, 1)",
            ],
        )
